#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 19 13:41:45 2017

@author: sebastian
"""
import sys
from PyQt5.QtWidgets import QApplication, QWidget, QPushButton, QStyleFactory, \
    QGridLayout, QListWidget, QVBoxLayout, QHBoxLayout, QLineEdit, QListWidgetItem, \
        QStatusBar, QProgressBar, QSplitter, QGroupBox, QCheckBox, QRadioButton, QComboBox, QLineEdit,\
            QLabel, QFormLayout, QSpinBox
#from PyQt5 import QStyleFactory
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtGui import QIcon


import numpy as np
import pyqtgraph as pg

import pyqtgraph.console
import matplotlib.image as mpimg
from os import listdir
from os.path import isfile, join
from camview.readimg import sbf, plifimg, sif
from camview.readlvfile import readlvfile
from camview.ImageViewPfaff import ImageViewPfaff
import pickle
import pyqtgraph.ptime as ptime
from camview.CCCView import CCCView
import os

class PLIFView(CCCView):
    def __init__(self,LoadFile=None):
        super().__init__(LoadFile)
        self.PLIFFileName=''
        self.readRaw=False
        self.fromFrame=0
        self.toFrame=-1
        self.profileMode=0
        self.profileData=None
        self.numBg=1

        self.dispInvert=False
        self.dispDiff=False
        self.divProfile=True
        self.averagedProfiles={}
        self.pathBoxes=[]

        self.showProfile=False
        self.UseAltBg=False
        self.initUI()

    def initUI(self):
        #self.path = './testdata/2021-03-17/'
        MainGrid=super(PLIFView,self).initUI()

        self.setWindowTitle('PLIF Image Viewerz')

        def LoadBgCheckClicked(yes):
            self.readRaw=yes
            ProfileGroup.setEnabled(not yes)    

        
        

        def fileNameChangedFunc(filename):
            if '.img' in filename:
                PLIFBgFileNameWidget.setVisible(False)
            self.PLIFFileName=self.path+filename

        PLIFGroup = QGroupBox('PLIF')
        PLIFGroup.setMaximumWidth(400)
        PLIFLayout=QVBoxLayout(PLIFGroup)
        
        PLIFFileNameWidget=QWidget()
        PLIFFileNameLayout=QHBoxLayout(PLIFFileNameWidget)
        PLIFFileNameLayout.setContentsMargins(0,0,0,0)
        PLIFFileNameBox=QComboBox()
        PLIFFileNameBox.setMinimumWidth(250)
        PLIFFileNameLayout.addWidget(QLabel('Data file'))
        PLIFFileNameLayout.addWidget(PLIFFileNameBox)
        PLIFLayout.addWidget(PLIFFileNameWidget)
        PLIFFileNameBox.currentTextChanged.connect(fileNameChangedFunc)

        PLIFBgFileNameWidget=QWidget()
        PLIFBgFileNameLayout=QHBoxLayout(PLIFBgFileNameWidget)
        PLIFBgFileNameLayout.setContentsMargins(0,0,0,0)
        PLIFBgFileNameBox=QComboBox()
        self.pathBoxes.append(PLIFFileNameBox)
        self.pathBoxes.append(PLIFBgFileNameBox)
        PLIFBgFileNameBox.setMinimumWidth(250)
        PLIFBgFileNameLayout.addWidget(QLabel('Background file'))
        PLIFBgFileNameLayout.addWidget(PLIFBgFileNameBox)
        PLIFLayout.addWidget(PLIFBgFileNameWidget)

        def SetAvg(value):
            self.LoadAverage=value

        def SetAltBg(value):
            self.UseAltBg=value

        AvgWidget=QWidget()
        AvgBox=QSpinBox()
        AvgBox.valueChanged.connect(SetAvg)
        AvgBox.setValue(10)
        AvgBox.setRange(0,100000)
        AvgLayout=QHBoxLayout(AvgWidget)
        AvgLayout.addWidget(QLabel('Average every'))
        AvgLayout.addWidget(AvgBox)
        AvgLayout.addWidget(QLabel('frames'))
        AvgLayout.setContentsMargins(0,0,0,0)
        PLIFLayout.addWidget(AvgWidget)

        LoadBgCheck=QCheckBox('Raw FPA data (sbf only)')
        LoadBgCheck.setChecked(False)

        def NumBgBoxFunc(value):
            self.numBg=value

        BgBox=QWidget()
        BgBoxLayout=QHBoxLayout(BgBox)
        AltBgCheck=QCheckBox('Use alt. bg.')
        AltBgCheck.toggled.connect(SetAltBg)
        BgBoxLayout.addWidget(AltBgCheck)
        BgBoxLayout.setContentsMargins(0,0,0,0)
        NumBgBox=QSpinBox()
        NumBgBox.setValue(1)
        LaserImgBox=QSpinBox()
        LaserImgBox.setValue(0)
        BgBoxLayout.addWidget(QLabel("Number of Bg images")) #number of bg shots
        BgBoxLayout.addWidget(NumBgBox)
        NumBgBox.valueChanged.connect(NumBgBoxFunc)

        def RangeRadioChange():
            for i,radio in enumerate(RangeRadios):
                if i == 1 and radio.isChecked():
                    FromBox.setEnabled(True)
                    ToBox.setEnabled(True)
                    self.fromFrame=FromBox.value()
                    self.toFrame=ToBox.value()
                if i == 0 and radio.isChecked():
                    FromBox.setEnabled(False)
                    ToBox.setEnabled(False)
                    self.fromFrame=0
                    self.toFrame=-1

        RangeWidget=QWidget()
        RangeLayout=QHBoxLayout(RangeWidget)
        RangeLayout.setContentsMargins(0,0,0,0)
        RangeRadios=[QRadioButton('All frames'),QRadioButton('Range')]
        for i,radio in enumerate(RangeRadios):
            RangeLayout.addWidget(radio)
            radio.toggled.connect(RangeRadioChange)

        FromBox=QSpinBox()
        ToBox=QSpinBox()
        FromBox.valueChanged.connect(RangeRadioChange)
        ToBox.valueChanged.connect(RangeRadioChange)

        ToBox.setRange(0,1000000)
        FromBox.setRange(0,1000000)

        RangeLayout.addWidget(FromBox)
        dashLabel=QLabel('to')
        dashLabel.setMaximumWidth(15)
        RangeLayout.addWidget(dashLabel)
        RangeLayout.addWidget(ToBox)
        PLIFLayout.addWidget(RangeWidget)
        PLIFLayout.addWidget(BgBox)
        PLIFLayout.addWidget(LoadBgCheck)
        LoadBgCheck.stateChanged.connect(LoadBgCheckClicked)

        LoadButton=QPushButton('Load Data File')
        PLIFLayout.addWidget(LoadButton)
        LoadButton.pressed.connect(self.loadFile)
        RangeRadios[0].setChecked(True)

        MainGrid.addWidget(PLIFGroup,2,1)

        ################ PROFILE ########################

        ProfileGroup = QGroupBox('Profile')
        ProfileGroup.setMaximumWidth(400)
        ProfileGrid=QGridLayout(ProfileGroup)
        UseSingleProfile=QRadioButton('Average single profile file',ProfileGroup)
        SingleProfileFiles=QComboBox()
        ProfileGrid.addWidget(SingleProfileFiles,1,0)

        UseAsRampProfile=QRadioButton('Use a profile with the same ramp as data',ProfileGroup)
        RampProfileFiles=QComboBox()
        UseMultipleProfiles=QRadioButton('Use multiple profile files',ProfileGroup)
        ProfileGrid.addWidget(RampProfileFiles,3,0)
        profileRadios=[UseSingleProfile,UseAsRampProfile,UseMultipleProfiles]

        #ProfileListW=QListWidget()
        ProfileListW=QWidget()
        ProfileListLayout=QVBoxLayout(ProfileListW)
        ProfileListListW=QListWidget()
        
        ProfileListLayout.addWidget(ProfileListListW)

        ProfileButtonsW=QWidget()
        ProfileButtonsWLayout=QHBoxLayout(ProfileButtonsW)
        AddProfileButton=QPushButton('Add Profile')
        RemProfileButton=QPushButton('Remove Profile')
        ProfileButtonsWLayout.addWidget(RemProfileButton)
        ProfileButtonsWLayout.addWidget(AddProfileButton)
        ProfileListLayout.addWidget(ProfileButtonsW)
        #ProfileListW.setMaximumHeight(100)
        ProfileGrid.addWidget(ProfileListW,7,0,1,1)
        self.pathBoxes.append(SingleProfileFiles)
        self.pathBoxes.append(RampProfileFiles)
        self.pathBoxes.append(ProfileListListW)
        

        def ProfileRadioChanged():
            fields=[SingleProfileFiles, RampProfileFiles, ProfileListW]
            for i,field in enumerate(fields):
                if profileRadios[i].isChecked():
                    field.setVisible(True)
                    self.profileMode=i
                    if i < 2:
                        self.ProfileFileName=self.path+field.currentText()
                    else:
                        self.ProfileFileName=[]
                        for index in range(ProfileListListW.count()):
                            if ProfileListListW.item(index).checkState()==2:
                                self.ProfileFileName.append(ProfileListListW.item(index))
                else:
                    field.setVisible(False)

        for i,radio in enumerate(profileRadios):
            ProfileGrid.addWidget(radio,i*2,0)
            radio.toggled.connect(ProfileRadioChanged)

        def startLoadProfile():
            ProfileRadioChanged()
            self.LoadProfile()
        #ProfileListListW.itemSelectionChanged.connect(ProfileRadioChanged)
        ProfileButton=QPushButton('Load Profile')
        ProfileGrid.addWidget(ProfileButton,10,0,1,1)
        ProfileButton.pressed.connect(startLoadProfile)
        MainGrid.addWidget(ProfileGroup,3,1)
        UseAsRampProfile.setChecked(True)

        ################ DISPLAY ########################
        def ProfileChanged(profile):
            self.divProfile=profile
            self.redrawSheet()

        def InvertChanged(invert):
            self.dispInvert=invert
            self.redrawSheet()

        def DiffChanged(diff):
            self.dispDiff=diff
            self.redrawSheet()

        def showProfileChanged(diff):
            self.showProfile=diff
            self.redrawSheet()

        DisplayGroup = QGroupBox('Display')
        DisplayGrid=QGridLayout(DisplayGroup)

        ProfileCheck=QCheckBox('Divide by profile/range')
        ProfileCheck.setChecked(True)
        InvertCheck=QCheckBox('Invert')
        DiffCheck=QCheckBox('Show Difference')
        showProfileCheck=QCheckBox('Show profile')

        InvertCheck.toggled.connect(InvertChanged)
        DiffCheck.toggled.connect(DiffChanged)
        ProfileCheck.toggled.connect(ProfileChanged)
        showProfileCheck.toggled.connect(showProfileChanged)


        DisplayGrid.addWidget(InvertCheck,0,0)
        DisplayGrid.addWidget(DiffCheck,0,1)
        DisplayGrid.addWidget(ProfileCheck,1,0)
        DisplayGrid.addWidget(showProfileCheck,1,1)

        MainGrid.addWidget(DisplayGroup,4,1)

        ProfileRadioChanged()
        self.fillPathBoxes(self.path)
        if self.LoadFileName is not None:
            pass
        # restore()
        self.show()

    def fillPathBoxes(self,path):
        pathBoxes=self.pathBoxes
        for box in pathBoxes:
            box.clear()

        try:
            files = [f for f in listdir(path) if isfile(join(path, f)) and 'maestro' not in str(f)]
        except FileNotFoundError:
            return
        for thisfile in files:
            if 'profile' in thisfile:
                pathBoxes[2].addItem(thisfile)
                pathBoxes[3].addItem(thisfile)
                ProfileListItem=QListWidgetItem(thisfile)
                ProfileListItem.setCheckState(False)
                pathBoxes[4].addItem(ProfileListItem)

            if 'img' in thisfile or 'sif' in thisfile:
                pathBoxes[0].addItem(thisfile)
                pathBoxes[1].addItem(thisfile)


    def ROIDragged(self,draggedROI):
        dataItem=None
        listItem=None
        invfactor=1
        if self.dispInvert:
            invfactor = -1
        for ROI in self.ROIList:
            if ROI[1]==draggedROI:
                dataItem=ROI[2]
                listItem=ROI[0]
        profileROI=1
        if self.divProfile and not self.readRaw:
            profile, coords = draggedROI.getArrayRegion(data=self.profileData, img=self.img, axes=(1,2),returnMappedCoords=True)
            if self.profileMode==0:
                profileROI=np.mean(profile)
            else:
                profileROI=np.mean(profile, axis=(1,2))
        data, coords = draggedROI.getArrayRegion(data=self.showData, img=self.img, axes=(1,2),returnMappedCoords=True)
        meanData=np.mean(data,axis=(1,2))
        xROILocString='X: {:.0f}-{:.0f}'.format(np.min(coords[0]),np.max(coords[0]))
        yROILocString='Y: {:.0f}-{:.0f}'.format(np.min(coords[1]),np.max(coords[1]))
        listItem.setText(xROILocString+', '+yROILocString)
        dataItem.setData(self.timeAxis,invfactor*meanData/profileROI)

    def LoadPLIF(self):
        if self.PLIFFileName.endswith('.sif'):
            file = sif(self.PLIFFileName)
        else:
            file = sbf(self.PLIFFileName,numbgframes=self.numBg)
            file.findPLIFframe()
        if self.readRaw:
            if self.UseAltBg:
                startimg = 1
            else:
                startimg = 0
            if self.numBg==1:    
                plifdata = file.readraw(self.fromFrame,self.toFrame)#[:,:,startimg::2]
            else:
                plifdata = file.readraw(self.fromFrame,self.toFrame)
        else:
            plifdata = plifimg.readimgav(file, self.fromFrame,self.toFrame, self.LoadAverage,altbg=self.UseAltBg)
        return plifdata.swapaxes(0,2).swapaxes(1,2),file

    def LoadProfile(self):
        if self.profileMode==2:
            for file in self.ProfileFileName:
                filename=self.path+file.text
            return

        if self.ProfileFileName.endswith('.sif'):
            file = sif(self.ProfileFileName)
        else:
            file = sbf(self.ProfileFileName,numbgframes=self.numBg)
        if self.profileMode==0:
            self.profileData=np.squeeze(plifimg.readimgav(file,0,-1,-1)).swapaxes(1,2)
        if self.profileMode==1:
            self.profileData=plifimg.readimgav(file,self.fromFrame,self.toFrame,self.LoadAverage).swapaxes(0,2)#.swapaxes(1,2)

        #return profiledata

    def getProfile(self,frame):
        if self.profileData is None:
            return 1
        if self.profileMode==0:
            return self.profileData
        if self.profileMode==1:
            TempProfile=self.profileData[int(frame)]
            #TempProfile[TempProfile<10]=10
            return TempProfile

    def loadFile(self):
        self.setWindowTitle('PLIF Image Viewer - '+ self.PLIFFileName.split('/')[-1])
        plifdata,file = self.LoadPLIF()
        self.showData = plifdata

        if self.readRaw:
            starttime=self.fromFrame//(self.numBg+1)//10
            if self.toFrame==-1:
                endtime=(file.numimgframes(rawcount=True)-1)//(self.numBg+1)//10
                print(file.numimgframes(True))
            else:
                endtime=self.toFrame//(self.numBg+1)//10
        else:
            starttime=self.fromFrame/10
            if self.toFrame==-1:
                endtime=file.numimgframes()/10
            else:
                endtime=self.toFrame/10
        self.timeAxis=np.linspace(starttime,endtime,np.shape(plifdata)[0])
        self.trendPlot.setXRange(starttime,endtime)
        self.trendScroll.setBounds((starttime,endtime))
        self.showData = np.swapaxes(self.showData, 2, 1)
        self.redrawSheet()

    def redrawSheet(self,sender=None):
        #print(sender)
        if self.dispInvert:
            invfactor = -1
        else:
            invfactor = 1
        if self.toFrame==-1:
            endframe=np.shape(self.showData)[0]
        else:
            endframe=self.toFrame-1

        if self.readRaw:
            framenr=int(self.trendScroll.getPos()[0]*(self.numBg+1)*10)-self.fromFrame
        else:
            framenr=int(10*self.trendScroll.getPos()[0]//(self.LoadAverage))-self.fromFrame//(self.LoadAverage)

        if framenr>np.shape(self.showData)[0]-1: #prevent overflow
            framenr=np.shape(self.showData)[0]-1
        profileFactor=1
        print('nr: '+str(framenr))
        if self.divProfile and not self.profileData is None and not self.readRaw:
            profileFactor=self.getProfile(framenr)

        if self.showProfile and not self.profileData is None and not self.readRaw:
            profileFactor=self.getProfile(framenr)
            self.img.setImage(invfactor/profileFactor,autoLevels=sender is not None)
            return

        if self.dispDiff:
            if framenr==0:
                framenr=1
            self.img.setImage(invfactor/profileFactor*(self.showData[framenr, :, :]-self.showData[framenr-1, :, :]),autoLevels=False)
        else:
            self.img.setImage(invfactor/profileFactor*self.showData[framenr, :, :],autoLevels=sender is not None)

class PLIFViewApp:
    def __init__(self):
        os.environ["QT_AUTO_SCREEN_SCALE_FACTOR"] = "1"
        app = QApplication(sys.argv)
        app.setApplicationName('LU PLIF Viewer')
        app.setWindowIcon(QIcon('PLIF-icon-256.png'))
        plifv = PLIFView()
        # sys.exit(app.exec_())
        app.exec_()

if __name__ == '__main__':
    app=PLIFViewApp()


# https://stackoverflow.com/questions/12459811/how-to-embed-matplotlib-in-pyqt-for-dummies



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Nov 19 16:30:14 2018

@author: sebastian
"""
def nm2invcm(nmin):
    return 1.0/nmin*1e7

def invcm2nm(invcmin):
    return 1.0/invcmin*1e7

def makeIRCamFunc(roomtemp=300): #this functions takes in a fraction of IR Cam intensity at HT/RT
    import numpy as np
    import matplotlib.pyplot as plt
    import scipy.signal
    from scipy.interpolate import interp1d

    lambdaspace=np.linspace(3000e-9,5000e-9,2000)

    theconstants=6.62607004e-34*3e8/lambdaspace/1.38064852e-23 #hc/lambda*Kb

    transmissionnr=.87
    transmission=np.zeros(2000)
    transmission[1260-70:1260+70]=transmissionnr

    intensities=np.zeros(1000)
    templist = range(200,1000)

    #roomtemp=300

    roomtempint=np.mean((1/lambdaspace**5/np.exp(theconstants/roomtemp)-1)*transmission)
    #plt.figure()

    for simtemps in templist: #0 K to 1000 K
        simintensity=(1/lambdaspace**5/np.exp(theconstants/simtemps)-1)
        intensities[simtemps]=np.mean(simintensity*transmission)

    #print(roomtempint)
    measuredintfrac=intensities/roomtempint
    #plt.figure()    

    #plt.xlim(0,10)
    #plt.ylim(200,400)
    return interp1d(measuredintfrac[200:1000],templist)

import matplotlib.pyplot as plt
from mpl_toolkits import axes_grid1

def add_colorbar(im, aspect=20, pad_fraction=0.5, **kwargs):
    """Add a vertical color bar to an image plot."""
    divider = axes_grid1.make_axes_locatable(im.axes)
    width = axes_grid1.axes_size.AxesY(im.axes, aspect=1./aspect)
    pad = axes_grid1.axes_size.Fraction(pad_fraction, width)
    current_ax = plt.gca()
    cax = divider.append_axes("right", size=width, pad=pad)
    plt.sca(current_ax)
    return im.axes.figure.colorbar(im, cax=cax, **kwargs)

def microshow(imdata,scale=1,colorbar=True,**kwargs):
    """Plot microscopy image"""
    from matplotlib_scalebar.scalebar import ScaleBar
    im=plt.imshow(imdata,**kwargs)
    plt.yticks(())
    plt.xticks(())
    scalebar = ScaleBar(scale,location='lower right',box_alpha=0.7) # 1 pixel = 0.2 meter
    plt.gca().add_artist(scalebar)
    if colorbar:
        cbar=add_colorbar(im)
        return im, cbar
    else:
        return None,im


#!/usr/bin/env python3
import imageio
import numpy as np
from scanf import scanf
import time
import os.path
#from readimg import sbf, plifimg
from camview.winspec import SpeFile

class imformat:
    """Function skeleton for image reader class
    """
    def __init__(self, filename):
        """[summary]

        Args:
            filename (string): image filename to open
        """
        self.f=open(filename,'rb')

    def imageType(self):
        """get the image type

        Returns:
            string: image type
        """
        return 'None'

    def getimgdata(self):
        """Returns width and height as a tuple.
        Returns:
        tuple: width, height
        """
        pass

    def numimgframes(self):
        """get number of frames in dataset
        Returns:
        int: number of frames
        """
        pass

    def readimg(self,startimg=0,stopimg=1):
        """[summary]

        Args:
            startimg (int, optional): [description]. Defaults to 0.
            stopimg (int, optional): [description]. Defaults to 1.
        """
        pass

class sif: #sif files made by the andor software. WIP, ask Sebastian how fucked up the Andor file format is....
    """
    A class that reads the contents and metadata of an Andor .sif file. Compatible with images as well as spectra.
    Exports data as numpy array or xarray.DataArray.
    Example: SIFFile('my_spectrum.sif').read_all()
    In addition to the raw data, SIFFile objects provide a number of meta data variables:
    :ivar x_axis: the horizontal axis (can be pixel numbers or wavelength in nm)
    :ivar original_filename: the original file name of the .sif f ile
    :ivar date: the date the file was recorded
    :ivar model: camera model
    :ivar temperature: sensor temperature in degrees Celsius
    :ivar exposuretime: exposure time in seconds
    :ivar cycletime: cycle time in seconds
    :ivar accumulations: number of accumulations
    :ivar readout: pixel readout rate in MHz
    :ivar xres: horizontal resolution
    :ivar yres: vertical resolution
    :ivar width: image width
    :ivar height: image height
    :ivar xbin: horizontal binning
    :ivar ybin: vertical binning
    :ivar gain: EM gain level
    :ivar vertical_shift_speed: vertical shift speed
    :ivar pre_amp_gain: pre-amplifier gain
    :ivar stacksize: number of frames
    :ivar filesize: size of the file in bytes
    :ivar m_offset: offset in the .sif file to the actual data
    
    #some code from https://github.com/lightingghost/sifreader/blob/master/sifreader/sifreader.py
    #self.f=open(self.filename,'rb')
    #Pixel number6
    # Counts12
    # Pixel number65541 1 2160 2560 1 12000 1 1239264000 103272
    # 65538 730 1780 1722 845 3 3 0
    """
    def __init__(self, filepath, verbose=False,offsetshift=0):
        self.filepath = filepath
        self.filename = filepath
        self._read_header(filepath, verbose, offsetshift)

    def __repr__(self):
        info = (('Original Filename', self.original_filename),
                ('Date', self.date),
                ('Camera Model', self.model),
                ('Temperature (deg.C)', '{:f}'.format(self.temperature)),
                ('Exposure Time', '{:f}'.format(self.exposuretime)),
                ('Cycle Time', '{:f}'.format(self.cycletime)),
                ('Number of accumulations', '{:d}'.format(self.accumulations)),
                ('Pixel Readout Rate (MHz)', '{:f}'.format(self.readout)),
                ("Horizontal Camera Resolution", '{:d}'.format(self.xres)),
                ("Vertical Camera Resolution", '{:d}'.format(self.yres)),
                ("Image width", '{:d}'.format(self.width)),
                ("Image Height", '{:d}'.format(self.height)),
                ("Horizontal Binning", '{:d}'.format(self.xbin)),
                ("Vertical Binning", '{:d}'.format(self.ybin)),
                ("EM Gain level", '{:f}'.format(self.gain)),
                ("Vertical Shift Speed", '{:f}'.format(self.vertical_shift_speed)),
                ("Pre-Amplifier Gain", '{:f}'.format(self.pre_amp_gain)),
                ("Stacksize", '{:d}'.format(self.stacksize)),
                ("Offset to Image Data", '{:f}'.format(self.m_offset)))
        desc_len = max([len(d) for d in list(zip(*info))[0]]) + 3
        res = ''
        for description, value in info:
            res += ('{:' + str(desc_len) + '}{}\n').format(description + ': ', value)

        res = super().__repr__() + '\n' + res
        return res

    def _read_header(self, filepath, verbose, offsetshift=0):
        f = open(filepath, 'rb')
        headerlen = 50
        spool = 0
        i = 0
        pxnumline=0
        while i < headerlen + spool:
            line = f.readline().strip()
            if verbose:
                print(str(i)+':'+str(f.tell())+': '+str(line))
            if i == 0:
                if line != b'Andor Technology Multi-Channel File':
                    f.close()
                    raise Exception('{} is not an Andor SIF file'.format(filepath))
            elif i == 2:
                tokens = line.split()
                self.temperature = float(tokens[5])
                self.date = time.strftime('%c', time.localtime(float(tokens[4])))
                self.exposuretime = float(tokens[12])
                self.cycletime = float(tokens[13])
                self.accumulations = int(tokens[15])
                self.readout = 1 / float(tokens[18]) / 1e6
                self.gain = float(tokens[21])
                self.vertical_shift_speed = float(tokens[41])
                self.pre_amp_gain = float(tokens[43])
            elif i == 3:
                self.model = line.decode('utf-8')
            elif i == 5:
                self.original_filename = line.decode('utf-8')
            elif i == 7:
                tokens = line.split()
                if len(tokens) >= 1 and tokens[0] == 'Spooled':
                    spool = 1
            elif line.startswith(b'Pixel number') and pxnumline==0:
                pxnumline=i
            elif i == pxnumline+2 and pxnumline>0:
                tokens=scanf('Pixel number%d %d %d %d %d %d %d %d %d', s=str(line), collapseWhitespace=False)
                self.yres = int(tokens[2])
                self.xres = int(tokens[3])
                self.stacksize = int(tokens[5])
            elif i == pxnumline+3 and pxnumline>0:
                tokens = scanf('%d %d %d %d %d %d %d', s=str(line), collapseWhitespace=False)
                if len(tokens) < 7:
                    raise Exception("Not able to read Image dimensions.")
                self.left = int(tokens[1])
                self.top = int(tokens[2])
                self.right = int(tokens[3])
                self.bottom = int(tokens[4])
                self.xbin = int(tokens[5])
                self.ybin = int(tokens[6])
            elif i>=pxnumline+4 and pxnumline > 0:# and str(line)==b'0':
                #'End of header, looking for start of data!'
                for i in range(self.stacksize):
                    f.readline()
                #'Header end: '+str(f.tell()))
                self.m_offset = f.tell()+offsetshift
                break;
            i += 1

        f.close()

        width = self.right - self.left + 1
        mod = width % self.xbin
        self.width = int((width - mod) / self.ybin)
        height = self.top - self.bottom + 1
        mod = height % self.ybin
        self.height = int((height - mod) / self.xbin)

    def imageType(self):
        return 'AndorSIF'

    def getimgdata(self): #returns width,height
        line=''
        self.f=open(self.filename,mode='r',errors='ignore')
        while True:
            line=self.f.readline()
            if line.startswith('Pixel number'):
                line=self.f.readline()
                line=self.f.readline()
                line=self.f.readline()
                out=scanf('%d %d %d %d %d %d %d', s=line, collapseWhitespace=False)
                leftPixel = out[1];
                topPixel = out[2];
                rightPixel = out[3];
                bottomPixel = out[4];
                vBin = out[6]
                hBin=vBin
                width = (rightPixel - leftPixel + 1)/hBin
                height = (topPixel - bottomPixel + 1)/vBin;
                self.f.close()
                return (int(width),int(height))
        self.f.close()
        return -1

    def numimgframes(self):
        return self.stacksize

    def readimg(self,startimg=0,stopimg=-1,offset=0):
        imageDims=self.getimgdata()
        NumFrames=self.numimgframes()
        if stopimg<0:
            stopimg=NumFrames
        self.f=open(self.filename,'rb')
        self.f.seek(4*startimg*imageDims[0]*imageDims[1]+self.m_offset+offset,0) #zyla file
        rs=np.zeros((imageDims[1],imageDims[0],stopimg-startimg),dtype='f')
        for i in range(stopimg-startimg):
            imageraw=self.f.read(4*imageDims[0]*imageDims[1])
            image=np.frombuffer(imageraw,dtype='f')
            rs[:,:,i]=np.reshape(image,(imageDims[1],imageDims[0]))
        #f.close()
        return rs

class lvsor: #SOR images created by labview, these are always 16 bits
    """Reads the custom format created by the Catalysis Group LabView program for SOR images.
    """
    def __init__(self, filename):
        self.f=open(filename,'rb')

    def imageType(self):
        return 'LVSor'

    def getimgdata(self): #returns width,height
        self.f.seek(14,0)
        width=np.frombuffer(self.f.read(2),dtype='h').item(0)
        height=np.frombuffer(self.f.read(2),dtype='h').item(0)
        return(width,height)

    def numimgframes(self):
        self.f.seek(-4,2)
        return(int.from_bytes(self.f.read(4),byteorder='little'))

    def readimg(self,startimg=0,stopimg=1):
        width=self.getimgdata()[0]
        height=self.getimgdata()[1]
        if startimg > self.numimgframes():
            return np.zeros(getimgdata(self),dtype='uint16')
        if stopimg > self.numimgframes():
            stopimg=stopimg=self.numimgframes()
        f=self.f
        if stopimg<0:
            stopimg=self.numimgframes()
        #header is 16 words aka 32 bytes
        f.seek(32+height*width*2*startimg,0)
        rs=np.zeros((height,width,stopimg-startimg),dtype='uint16')
        for i in range(stopimg-startimg):
            image=f.read(height*width*2)
            imdb=np.frombuffer(image,dtype='uint16')
            rs[:,:,i]=np.reshape(imdb,(height,width))
        #f.close()
        return rs

class mptif16:
    """Class to read multi-page TIFF files as produced by the Thorcam software by Thorlabs. One dataset can be split into many subfiles.
    """
    def __init__(self, filename):
        self.filename = filename
        self.f=imageio.get_reader(self.filename,'tiff','I')
        self.subfiles=[self]
        numfiles=1
        while True:
            if os.path.isfile(str(filename).split('.tif')[0]+'_'+str(numfiles)+'.tif'):
                self.subfiles.append(mptif16(str(self.filename).split('.tif')[0]+'_'+str(numfiles)+'.tif'))
                numfiles+=1
            else:
                break
        self.numfiles=numfiles
        self.imagedata=self.f.get_next_data()

    def getnumfiles(self):
        """Get number of subfiles in dataset.

        Returns:
            int: Number of subfiles
        """
        return self.numfiles

    def imageType(self):
        return 'MPTIF'

    def numimgframes(self,onlythis=False):
        """Get number of frames in the dataset.

        Args:
            onlythis (bool, optional): Whether to consider subfiles. Defaults to False.

        Returns:
            int: nunber of frames
        """
        if self.numfiles==1 or onlythis:
            return self.f.get_length()
        else:
            total=self.f.get_length()
            for i in range(self.getnumfiles()-1):
                nextfile=mptif16(str(self.filename).split('.tif')[0]+'_'+str(i+1)+'.tif')
                total=total+nextfile.numimgframes(onlythis=True)
            return total

    def numimgframesperfile(self):
        """Gets number of frames per subfile.

        Returns:
            int: number of frames
        """
        if self.numfiles==1:
            return self.f.get_length()
        else:
            total=[]
            total.append(self.f.get_length())
            for i in range(self.getnumfiles()-1):
                nextfile=mptif16(str(self.filename).split('.tif')[0]+'_'+str(i+1)+'.tif')
                total.append(nextfile.numimgframes(onlythis=True))
            return total

    def getimgdata(self): #returns width,height
        """Get image dimensions

        Returns:
            tuple: (width,height)
        """
        return np.shape(self.imagedata)

    def whichfile(self,frame):
        """Check which subfile contains a certain frame.

        Args:
            frame (int): The frame to check

        Returns:
            int: subfile number with that frame.
        """
        if self.numfiles==1:
            return 0
        framenums=self.numimgframesperfile()
        imgfile=0
        currframe=frame
        while True:
            currframe-=framenums[imgfile]
            if currframe<=0:
                return imgfile,framenums[imgfile]+currframe
            imgfile+=1
        return 0

    def readimg(self,startimg=0,stopimg=-1,onlythis=False):
        """Read image sequence. Will automatically consider subfiles.

        Args:
            startimg (int, optional): Read from this image number. Defaults to 0 (beginning of file).
            stopimg (int, optional): Read to this image number, -1 reads entire file. Defaults to -1 (eof).
            onlythis (bool, optional): Ignore subfiles. Defaults to False.

        Returns:
            numpy array: Array with all the images. Axis order is (x,y,t).
        """
        imageDims=self.getimgdata()
        if stopimg<0:
            if onlythis:
                stopimg=self.numimgframes(onlythis=True)
            else:
                 stopimg=self.numimgframes()

        if self.numfiles>1 and not onlythis: #not a subfile
            rs=np.empty((imageDims[0],imageDims[1],0),dtype='uint16')
            startdata=self.whichfile(startimg)
            stopdata=self.whichfile(stopimg)
            globalindex=startimg
            currentfile=startdata[0]
            localstartindex=startdata[1]
            while currentfile<stopdata[0]:
                rs=np.append(rs,self.subfiles[currentfile].readimg(localstartindex,-1,True),2)
                localstartindex=0
                currentfile+=1
            if currentfile==stopdata[0]:
                rs=np.append(rs,self.subfiles[currentfile].readimg(localstartindex,stopdata[1],True),2)
        else:
            self.f.set_image_index(startimg)
            rs=np.zeros((imageDims[0],imageDims[1],stopimg-startimg),dtype='uint16')
            for i in range(stopimg-startimg):
                rs[:,:,i]=self.f.get_next_data()
                #print('.', end='',flush=True)
            #f.close()
        return rs

class sbf:
    """Class to read files made by the WinIR 3.x software by Santa Barbara Focalplane.
    """
    def __init__(self, filename,numbgframes=1):
        self.numbgframes=numbgframes
        self.filename = filename
        self.f=open(self.filename,'rb')

    def imageType(self):
        return 'SBF'

    def numimgframes(self, rawcount=False):
        self.f.seek(42,0)
        if rawcount:
            return(int.from_bytes(self.f.read(4),byteorder='little'))
        else:
            return(int.from_bytes(self.f.read(4),byteorder='little'))//(self.numbgframes+1)

    def getimgdata(self): #returns width,height
        return (256,256)

    def findPLIFframe(self):
        """This function checks the PLIF data in three places (25% in, 50% in and 75% in.
        It then tries to find which frame is the one with PLIF data in it (and not bg)

        Returns:
            int: first frame with laser
        """
        cycleframes=self.numbgframes+1
        fractions=np.array([.25,.5,.75]) #where to check
        startframes=self.numimgframes(True)*fractions//cycleframes*cycleframes
        #now load 10 cycles at every spot and find the maximum of each cycle
        allmax=[]
        for startframe in startframes:
            for cycle in range(10):
                cycledata=self.readraw(int(startframe+cycle*cycleframes),int(startframe+(cycle*cycleframes)+cycleframes))
                roi=np.mean(cycledata[100:120,100:120,:],axis=(0,1))
                allmax.append(np.argmax(roi))
        return np.argmax(np.bincount(np.array(allmax)))


    def readraw(self,startimg=0,stopimg=1):
        stopimg=stopimg
        if startimg > self.numimgframes(rawcount=True):
            return np.zeros((256,256),dtype='h')
        if stopimg > self.numimgframes(rawcount=True):
            stopimg=stopimg=self.numimgframes(rawcount=True)
        f=self.f
        if stopimg<0:
            stopimg=self.numimgframes(rawcount=True)
        #f.seek(512,0)
        f.seek(512+65536*2*startimg,0)
        rs=np.zeros((256,256,stopimg-startimg),dtype='h')
        for i in range(stopimg-startimg):
            image=f.read(65536*2)
            imdb=np.frombuffer(image,dtype='h')
            rs[:,:,i]=np.reshape(imdb,(256,256))
        #f.close()
        return rs
    
    def readimg(self,startimg=0,stopimg=-1,altbg=False,raw=False):
        """Reads the SBF format images

        Args:
            startimg (int, optional): Read from this image number. Defaults to 0 (beginning of file).
            stopimg (int, optional): Read to this image number, -1 reads entire file. Defaults to -1 (eof).
            altbg (bool, optional): Use the background before the data frame rather than that after. Defaults to False.
            numbgframes (int, optional): Number of background frames between PLIF frames. Defaults to 1.
            plifframe (int, optional): Which frame is the PLIF frame in each frame cycle. Defaults to 0.

        Returns:
            numpy array: Array with all the images. Axis order is (x,y,t)
        """
        if raw:
            return self.readraw(startimg,stopimg)

        cycleframes=self.numbgframes+1 #number of frames in 100 ms, usually 2 unless hifps on IR camera
        skipframes=cycleframes-2 #background frames to skip 
        #initialskip = plifframe%cycleframes #make sure the skip is periodic
        initialskip = self.findPLIFframe() #make sure the skip is periodic
        if altbg:
            plifframe=plifframe-1
        if startimg > self.numimgframes():
            return np.zeros((256,256),dtype='h')
        if stopimg > self.numimgframes():
            stopimg=self.numimgframes()//cycleframes
        f=self.f
        if stopimg<0:
            stopimg=self.numimgframes()//cycleframes
        #skip 512 byte header, then skip one cycle/frame, then skip the initialskip
        f.seek(512+65536*2*cycleframes*startimg+65536*2*initialskip,0)
        rs=np.zeros((256,256,stopimg-startimg),dtype='h')
        for i in range(stopimg-startimg):
            if altbg:
                imagebg=f.read(65536*2)
                image=f.read(65536*2)
            else:
                image=f.read(65536*2)
                imagebg=f.read(65536*2)
            imdb=np.subtract(np.frombuffer(image,dtype='h'),np.frombuffer(imagebg,dtype='h'))
            rs[:,:,i]=np.reshape(imdb,(256,256))
            f.seek(65536*2*skipframes,1)
        #f.close()
        return rs

class plifimg:
    """This class contains static utility functions that are nice to have when handling PLIF images.
    """
    @staticmethod
    def readimgav(img,startimg=0,stopimg=-1,numavg=10,status=True,binning=None,forcefunc=False,**kwargs):
        """Reads an image file and averages a number of frames in time. This conserves memory when reading large files.

        Args:
            img (imformat): Image file to read
            startimg (int, optional): Read from this image, default is start of file. Defaults to 0.
            stopimg (int, optional): Read to this frame, -1 will read the entire file. Defaults to -1.
            numavg (int, optional): Number of images to average. Defaults to 10.
            status (bool, optional): Verbose status prints. Defaults to True.
            binning ([type], optional): Binning in space. Not implemented. Defaults to None.
            forcefunc (bool, optional): Force the use of the averaging function even for numavg=1. Defaults to False.

        Returns:
            numpy array: Array with all the images. Axis order is (x,y,t)
        """
        if 'raw' in kwargs:
            numframes=img.numimgframes(rawcount=kwargs.get("raw"))
        else:
            numframes=img.numimgframes()
        if stopimg<0 or \
         (img.imageType()=='SBF' and stopimg>numframes//2) or \
         stopimg>numframes:  #if -1, read to end
            stopimg=numframes
        if numavg==1 and not forcefunc: #1 average one = no average
            rs=img.readimg(startimg,stopimg,**kwargs)
            return rs
        if numavg==-1: #if -1, average all
            numavg=numframes
        imageDims=img.getimgdata()
        if img.imageType()=='AndorSIF' or img.imageType()=='LVSor':
            rs=np.zeros((imageDims[1],imageDims[0],(stopimg-startimg)//numavg),dtype='float64')
        else:
            rs=np.zeros((imageDims[0],imageDims[1],(stopimg-startimg)//numavg),dtype='float64')

        for i in range((stopimg-startimg)//numavg):
            frame=i*numavg+startimg
            if (status):
                print('reading '+str(frame)+' to '+str(frame+numavg-1))
            temp=img.readimg(frame,frame+numavg-1,**kwargs)
            rs[:,:,i]=np.nanmean(temp,axis=2)
        return rs
    
    @staticmethod
    def makeProfileFunction(profileMeta,labView=None,plifLength=None,numAvg=10):
        """[summary]

        Args:
            profileMeta ([type]): [description]
            labView ([type], optional): [description]. Defaults to None.
            plifLength ([type], optional): [description]. Defaults to None.
            numAvg (int, optional): [description]. Defaults to 10.

        Returns:
            [type]: [description]
        """
        from readlvfile2 import readlvfile2dict
        from scipy.interpolate import interp1d
        if np.size(profileMeta)==1: #assume single filename, just load and average all
            if profileMeta.endswith('sif'):
                return np.abs(np.squeeze(plifimg.readimgav(sif(profileMeta),numavg=-1,status=False)))
            else:
                return np.abs(np.squeeze(plifimg.readimgav(sbf(profileMeta),numavg=-1,status=False)))


        if np.size(profileMeta)==2 and profileMeta[1]=='ramp': #assume its a ramp, just return it with x avg
            if profileMeta[0].endswith('sif'):
                return np.swapaxes(np.swapaxes(np.abs(np.squeeze(plifimg.readimgav(sif(profileMeta[0]),numavg=numAvg,status=False))), 0, 2), 1, 2)
            else:
                return np.swapaxes(np.swapaxes(np.abs(np.squeeze(plifimg.readimgav(sbf(profileMeta[0]),numavg=numAvg,status=False))), 0, 2), 1, 2)

        else: #assume multiple profiles in a tuple.
            profileData=np.zeros([len(profileMeta[0]),256,256])
            for prf in range(len(profileMeta[0])):
                profileData[prf,:,:]=np.abs(np.squeeze(plifimg.readimgav(sbf(profileMeta[0][prf]),numavg=-1,status=False)))
            lvInfo=readlvfile2dict(labView)
            profileValueFunction=interp1d(np.linspace(profileMeta[1][1][0],profileMeta[1][1][1],len(profileMeta[0])),profileData,axis=0,fill_value='extrapolate',bounds_error=False)
            valueFunction=interp1d(readlvfile2dict(labView)['times'],readlvfile2dict(labView)[profileMeta[1][0]],fill_value='extrapolate',bounds_error=False)
            timeFrame=np.linspace(0,int(np.max(readlvfile2dict(labView)['times'])))
            #plifLength=sbf(plifFile).numimgframes()//20 #length in seconds
            #plifTimeValues=interp1d(range(plifLength),valueFunction(range(plifLength)))
            profileFunction=interp1d(timeFrame,profileValueFunction(valueFunction(timeFrame)),axis=0)
            if (plifLength==None):
                return profileFunction
            else:
                plifSpace=np.arange(0,plifLength,numAvg/10)
                return profileFunction(plifSpace)

class spe(SpeFile):

    def __init__(self, filename):
        super().__init__(filename)
        self.f=open(filename,'rb')

    def imageType(self):
        return 'SPE'

    def getimgdata(self): #returns width,height
        return(self.header.xdim,self.header.ydim)

    def numimgframes(self):
        return(self.header.NumFrames)

    def readimg(self,startimg=0,stopimg=1):
        width=self.getimgdata()[0]
        height=self.getimgdata()[1]
        if startimg > self.numimgframes():
            return np.zeros(self.getimgdata(),dtype='uint16')
        if stopimg > self.numimgframes():
            stopimg=stopimg=self.numimgframes()
        f=self.f
        if stopimg<0:
            stopimg=self.numimgframes()
        #header is 16 words aka 32 bytes
        f.seek(4100+height*width*2*startimg,0)
        rs=np.zeros((height,width,stopimg-startimg),dtype='uint16')
        for i in range(stopimg-startimg):
            image=f.read(height*width*2)
            imdb=np.frombuffer(image,dtype='uint16')
            rs[:,:,i]= np.fliplr(np.rot90(np.reshape(imdb,(height,width)),k = -2))
        #f.close()
        return rs

class speMany:
    def __init__(self, filename,fromFrame,toFrame,LoadAverage):
        self.file = filename
        self.data = []
        self.len = 0 
        self.fromFrame = fromFrame
        self.toFrame = toFrame
        self.LoadAverage = LoadAverage
        for fname in os.listdir(self.file):
            if fname.endswith(".SPE"):
                self.xdim = spe(os.path.join(self.file,fname)).getimgdata()[0] 
                self.ydim = spe(os.path.join(self.file,fname)).getimgdata()[1] 
                self.len += spe(os.path.join(self.file,fname)).numimgframes()
        # self.data = self.data.reshape((self.xdim,self.ydim,len(self.data)))
 
          
    def imageType(self):
        return 'SPE'

    def getimgdata(self): #returns width,height
        return(self.xdim,self.ydim)

    def numimgframes(self):
        return(self.len)

    def readimg(self,startimg=0,stopimg=1):
        data = np.zeros((self.len,self.xdim,self.ydim))
        temp = 0
        j = 0
        for fname in os.listdir(self.file):
            if fname.endswith(".SPE"):
                file = spe(os.path.join(self.file,fname))
                temp = plifimg.readimgav(file,self.fromFrame,self.toFrame,self.LoadAverage).swapaxes(0,2).swapaxes(1,2)
                i = 0
                for i in range(temp.shape[0]):
                    data[j,:,:] = temp[i,:,:]
                    j += 1
                    i += 1
        
        return data

        # return self.data.reshape((self.xdim,self.ydim,len(self.data)))




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 19 13:41:45 2017

@author: sebastian
"""
import sys
from PyQt5.QtWidgets import QApplication, QWidget, QPushButton, QStyleFactory, \
    QGridLayout, QListWidget, QVBoxLayout, QHBoxLayout, QLineEdit, QListWidgetItem, \
        QStatusBar, QProgressBar, QSplitter, QGroupBox, QCheckBox, QRadioButton, QComboBox, QLineEdit,\
            QLabel, QFormLayout, QSpinBox, QFileDialog
#from PyQt5 import QStyleFactory
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtGui import QIcon


import numpy as np
import pyqtgraph as pg
import pyqtgraph.parametertree.parameterTypes as pTypes
from pyqtgraph.parametertree import Parameter, ParameterTree, ParameterItem, registerParameterType
import pyqtgraph.console
import matplotlib.image as mpimg
from os import listdir
from os.path import isfile, join
from camview.readimg import sbf, plifimg, sif, lvsor, mptif16, spe
from camview.readlvfile import readlvfile
from camview.ImageViewPfaff import ImageViewPfaff
import pickle
import pyqtgraph.ptime as ptime
from camview.CCCView import CCCView

class SORView(CCCView):
    def __init__(self,LoadFile=None):
        super().__init__(LoadFile)

        self.PLIFFileName=''
        self.fromFrame=0
        self.toFrame=-1
        self.LoadFps=10

        self.divData=1
        self.dispInvert=False
        self.dispDiff=False
        self.divProfile=False
        self.showData=np.random.rand(256,256,256)
        self.initUI()

    def cmapClicked(self, b=None):
        """onclick handler for our custom entries in the GradientEditorItem's context menu"""
        act = self.sender()
        self.hist.gradient.restoreState(self.mplCmaps[act.name])
        if act.name == 'turbo':
            self.hist.gradient.showTicks(show=False)

    def initUI(self):
        self.setWindowTitle('SOR Image Viewer')
        MainGrid=super(SORView,self).initUI()
        print(MainGrid.itemAtPosition(0,1))
        # self.status = QStatusBar(self)
        # self.statusProgress = QProgressBar()
        # self.status.addWidget(self.statusProgress)
        # MainGrid.addWidget(self.status,20,0,1,2)
        # self.status.showMessage('TestStatus')


        def divRegionChanged():
            if self.divProfile:
                fromframe=int(divRegion.getRegion()[0]/self.LoadAverage*10)-1
                toframe=int(divRegion.getRegion()[1]/self.LoadAverage*10)-1
                print(fromframe)
                print(toframe)
                self.divData=np.mean(self.showData[fromframe:toframe,:,:],axis=0)
                self.redrawSheet()

        divRegion=pg.LinearRegionItem()
        self.trendPlot.addItem(divRegion)
        divRegion.sigRegionChangeFinished.connect(divRegionChanged)
        #self.path = './testdata/2021-03-17/'
        if len(sys.argv) > 1:
            self.path = sys.argv[1]
        
        def fileNameChangedFunc(filename):
            self.PLIFFileName=self.path+filename
            print(self.PLIFFileName)

        PLIFGroup = QGroupBox('SOR')
        PLIFGroup.setMaximumWidth(400)
        PLIFLayout=QVBoxLayout(PLIFGroup)
        
        PLIFFileNameWidget=QWidget()
        PLIFFileNameLayout=QHBoxLayout(PLIFFileNameWidget)
        PLIFFileNameLayout.setContentsMargins(0,0,0,0)
        self.PLIFFileNameBox=QComboBox()
        self.PLIFFileNameBox.setMinimumWidth(250)
        PLIFFileNameLayout.addWidget(QLabel('Data file'))
        PLIFFileNameLayout.addWidget(self.PLIFFileNameBox)
        PLIFLayout.addWidget(PLIFFileNameWidget)
        self.PLIFFileNameBox.currentTextChanged.connect(fileNameChangedFunc)

        def SetAvg(value):
            self.LoadAverage=value
        
        def SetFps(value):
            self.LoadFps=value

        AvgWidget=QWidget()
        AvgBox=QSpinBox()
        AvgBox.valueChanged.connect(SetAvg)
        AvgBox.setValue(10)
        AvgBox.setRange(0,100000)
        AvgLayout=QHBoxLayout(AvgWidget)
        AvgLayout.addWidget(QLabel('Average every'))
        AvgLayout.addWidget(AvgBox)
        AvgLayout.addWidget(QLabel('frames'))
        AvgLayout.setContentsMargins(0,0,0,0)
        PLIFLayout.addWidget(AvgWidget)

        FpsWidget=QWidget()
        FpsBox=QSpinBox()
        FpsBox.setRange(1,400)
        FpsBox.valueChanged.connect(SetFps)
        FpsLayout=QHBoxLayout(FpsWidget)
        FpsLayout.addWidget(QLabel('Footage is'))
        FpsLayout.addWidget(FpsBox)
        FpsLayout.addWidget(QLabel('fps'))
        FpsLayout.setContentsMargins(0,0,0,0)
        PLIFLayout.addWidget(FpsWidget)

        LoadBgCheck=QCheckBox('Load IR background (sbf only)')
        LoadBgCheck.setChecked(False)

        def RangeRadioChange():
                for i,radio in enumerate(RangeRadios):
                    if i == 1 and radio.isChecked():
                        FromBox.setEnabled(True)
                        ToBox.setEnabled(True)
                        self.fromFrame=FromBox.value()
                        self.toFrame=ToBox.value()
                    if i == 0 and radio.isChecked():
                        FromBox.setEnabled(False)
                        ToBox.setEnabled(False)
                        self.fromFrame=0
                        self.toFrame=-1

        RangeWidget=QWidget()
        RangeLayout=QHBoxLayout(RangeWidget)
        RangeLayout.setContentsMargins(0,0,0,0)
        RangeRadios=[QRadioButton('All frames'),QRadioButton('Range')]
        for i,radio in enumerate(RangeRadios):
            RangeLayout.addWidget(radio)
            radio.toggled.connect(RangeRadioChange)

        FromBox=QSpinBox()
        ToBox=QSpinBox()
        FromBox.valueChanged.connect(RangeRadioChange)
        ToBox.valueChanged.connect(RangeRadioChange)

        ToBox.setRange(0,1000000)
        FromBox.setRange(0,1000000)

        RangeLayout.addWidget(FromBox)
        dashLabel=QLabel('to')
        dashLabel.setMaximumWidth(15)
        RangeLayout.addWidget(dashLabel)
        RangeLayout.addWidget(ToBox)
        PLIFLayout.addWidget(RangeWidget)
        PLIFLayout.addWidget(LoadBgCheck)

        LoadButton=QPushButton('Load Data File')
        PLIFLayout.addWidget(LoadButton)
        LoadButton.pressed.connect(self.loadFile)
        RangeRadios[0].setChecked(True)

        MainGrid.addWidget(PLIFGroup,2,1)

        ################ DISPLAY ########################
        def ProfileChanged(profile):
            self.divProfile=profile
            self.redrawSheet()

        def InvertChanged(invert):
            self.dispInvert=invert
            self.redrawSheet()

        def DiffChanged(diff):
            self.dispDiff=diff
            self.redrawSheet()

        DisplayGroup = QGroupBox('Display')
        DisplayGrid=QGridLayout(DisplayGroup)

        ProfileCheck=QCheckBox('Divide by region')
        ProfileCheck.setChecked(False)
        InvertCheck=QCheckBox('Invert')
        DiffCheck=QCheckBox('Show Difference')

        InvertCheck.toggled.connect(InvertChanged)
        DiffCheck.toggled.connect(DiffChanged)
        ProfileCheck.toggled.connect(ProfileChanged)


        DisplayGrid.addWidget(InvertCheck,0,0)
        DisplayGrid.addWidget(DiffCheck,0,1)
        DisplayGrid.addWidget(ProfileCheck,1,0)

        MainGrid.addWidget(DisplayGroup,4,1)

        self.fillPathBoxes(self.path)

    def fillPathBoxes(self,path):
        self.PLIFFileNameBox.clear()
        try:
            files = [f for f in listdir(path) if isfile(join(path, f)) and 'maestro' not in str(f)]
        except FileNotFoundError:
            return
        for thisfile in files:
            if 'sor' in thisfile or 'sif' in thisfile or 'tif' in thisfile:
                self.PLIFFileNameBox.addItem(thisfile)

    def LoadPLIF(self):
        plifdata=None
        if self.PLIFFileName.endswith('.sif'):
            file = sif(self.PLIFFileName)
        if self.PLIFFileName.endswith('.SPE') or self.PLIFFileName.endswith('.spe'):
            file = spe(self.PLIFFileName)
        if self.PLIFFileName.endswith('.tif'):
            file = mptif16(self.PLIFFileName)
        else:
            print(self.PLIFFileName)
            file = lvsor(self.PLIFFileName)

        plifdata = plifimg.readimgav(file, self.fromFrame,self.toFrame, self.LoadAverage)
        return plifdata.swapaxes(0,2).swapaxes(1,2),file

    def loadFile(self):
        self.setWindowTitle('SOR Image Viewer - '+ self.PLIFFileName.split('/')[-1])
        plifdata,file = self.LoadPLIF()
        self.showData = plifdata
        #if self.p['PLIF', 'ProfileDivision'] and not self.p['PLIF', 'ReadRaw']:
        #    profiledata=self.LoadProfile()
        #    self.showData=np.true_divide(plifdata,profiledata)
        #    self.showData[self.showData < -10] = 20
        #    self.showData[self.showData > 20] = -10
        starttime=self.fromFrame/self.LoadAverage/self.LoadFps
        if self.toFrame>-1:
            endtime=self.toFrame/self.LoadAverage/self.LoadFps#-(1/self.LoadAverage*self.LoadFps)
        else:
            endtime=file.numimgframes()//10
        self.timeAxis=np.linspace(starttime,endtime,np.shape(plifdata)[0])
        self.trendPlot.setXRange(starttime,endtime)
        self.trendScroll.setBounds((starttime,endtime))
        self.showData = np.swapaxes(self.showData, 2, 1)

        self.redrawSheet()

    def redrawSheet(self,sender=None):
        #print(sender)
        if self.dispInvert:
            invfactor = -1
        else:
            invfactor = 1
        if self.toFrame==-1:
            endframe=np.shape(self.showData)[0]
        else:
            endframe=self.toFrame-1

        framenr=int(self.trendScroll.getPos()[0]/self.LoadAverage*self.LoadFps)-self.fromFrame
        print(framenr)
        profileFactor=1

        if self.divProfile and self.divData != 1:
            profileFactor=self.divData

        if self.dispDiff:
            if framenr==0:
                framenr=1
            self.img.setImage(invfactor/profileFactor*(self.showData[framenr, :, :]-self.showData[framenr-1, :, :].astype(np.intc)),autoLevels=False)
        else:
            self.img.setImage(invfactor/profileFactor*self.showData[framenr, :, :],autoLevels=False)

#%%
class SORViewApp:
    def __init__(self):
        import os
        os.environ["QT_AUTO_SCREEN_SCALE_FACTOR"] = "1"
        app = QApplication(sys.argv)
        app.setApplicationName('LU SOR Viewer')
        app.setWindowIcon(QIcon('PLIF-icon-256.png'))
        plifv = SORView()
        # sys.exit(app.exec_())
        app.exec_()

if __name__ == '__main__':
    app=SORViewApp()


# https://stackoverflow.com/questions/12459811/how-to-embed-matplotlib-in-pyqt-for-dummies



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Aug 14 11:42:36 2018

@author: sebastian
"""

import matplotlib.pyplot as plt
import matplotlib.patches as patches
import numpy as np;
from mpl_toolkits.axes_grid1 import make_axes_locatable
#top bottom, left right
def catashow(image,cataloc,vmin=0, vmax=1,cmap='plasma',crop=[0,0,0,0],
             catamm=6,mmaxis=True,colorbar=True,giveback=False,curved=None):
    dim=np.shape(image)
    
    #default should not crop at all
    if crop[1]==0:
        crop[1]=dim[0]
    if crop[3]==0:
        crop[3]=dim[1]
   

    imout=plt.imshow(image,vmin=vmin,vmax=vmax,cmap=cmap,interpolation='none')

    scale=(cataloc[1]-cataloc[0])/catamm
    
    catathickness=int(1*scale)
    
    #put in rects, set limits    
    catarect=patches.Rectangle((cataloc[0],crop[1]-1),cataloc[1]-cataloc[0],catathickness,fc='w')
    bottomrect=patches.Rectangle((crop[2],crop[1]-1),crop[3]-crop[2],catathickness,fc='k')
    plt.gca().add_patch(bottomrect)
    plt.gca().add_patch(catarect)
    if (curved != None):
        cataellipse=patches.Ellipse((cataloc[0]+(cataloc[1]-cataloc[0])/2,crop[1]),cataloc[1]-cataloc[0],curved,fc='w')
        plt.gca().add_patch(cataellipse)
    
    if mmaxis:
        hcatapxsize=(cataloc[1]-cataloc[0])/2
        catacenter=int((cataloc[0]+cataloc[1])/2)
        labelpositions=range(catacenter+int(-2*hcatapxsize),catacenter+int(3*hcatapxsize),int(hcatapxsize))
        plt.xticks(labelpositions,(-catamm,-catamm/2,0,catamm/2,catamm))
        plt.xlabel('$x$ [mm]')
        plt.ylabel('$h$ [mm]')
        plt.yticks((range(crop[1]-1,crop[0]-1,-int(scale))),range(0,10,1))
    
    plt.ylim((crop[1]+catathickness-1,crop[0]))
    plt.xlim((crop[2],crop[3]-1))

    if colorbar:
        #add colorbar to a new axis to the rihgt
        ax = plt.gca()
        divider = make_axes_locatable(ax)
        cax = divider.append_axes("right", size="5%", pad=0.05)
        plt.colorbar(cax=cax)
        plt.sca(ax)
    
    if giveback:
        return imout,(bottomrect,catarect)
#catashow(np.polyval(calibfit,np.mean(divplif[1000:1400,100:200,:],axis=0)),
#         vmin=100,vmax=230,cmap='plasma',crop=[20,0,40,230],cataloc=(97,175))


    



#!/usr/bin/env python3
import pyqtgraph as pg
import numpy as np

from PyQt5 import QtCore, QtGui, QtWidgets
from pyqtgraph.widgets.PlotWidget import *
from pyqtgraph.imageview import *
from pyqtgraph.widgets.GraphicsLayoutWidget import GraphicsLayoutWidget
from pyqtgraph.graphicsItems.GradientEditorItem import addGradientListToDocstring
from pyqtgraph.widgets.GraphicsView import GraphicsView
import matplotlib.cm
import collections
QAPP = None

class ImageViewPfaff(pg.ImageView):
    images = []
    def image(*args, **kargs):
        """
        Create and return an :class:`ImageWindow <pyqtgraph.ImageWindow>`
        (this is just a window with :class:`ImageView <pyqtgraph.ImageView>` widget inside), show image data inside.
        Will show 2D or 3D image data.
        Accepts a *title* argument to set the title of the window.
        All other arguments are used to show data. (see :func:`ImageView.setImage() <pyqtgraph.ImageView.setImage>`)
        """
        mkQApp()
        w = ImageWindow(*args, **kargs)
        images.append(w)
        w.show()
        return w

    def buildMenu(self):
        super(ImageViewPfaff, self).buildMenu()

        self.trendAction = QtGui.QAction("Trend", self.menu)
        self.trendAction.setCheckable(True)
        self.trendAction.toggled.connect(self.trendToggled)
        self.menu.addAction(self.trendAction)

    def __init__(self,additionalCmaps=[], setColormap=None, **kargs):
        super(ImageViewPfaff, self).__init__(**kargs)
        self.trendroi=pg.LineROI([0, 60], [20, 80], width=5)
        self.trendroi.setZValue(30)
        self.view.addItem(self.trendroi)
        self.trendroi.hide()

        self.gradientEditorItem = self.ui.histogram.item.gradient

        self.activeCm = "grey"
        self.mplCmaps = {}

        if len(additionalCmaps) > 0:
            self.registerCmap(additionalCmaps)

        if setColormap is not None:
            self.gradientEditorItem.restoreState(setColormap)

    def registerCmap(self, cmapNames):
        """ Add matplotlib cmaps to the GradientEditors context menu"""
        self.gradientEditorItem.menu.addSeparator()
        savedLength = self.gradientEditorItem.length
        self.gradientEditorItem.length = 100

        # iterate over the list of cmap names and check if they're avaible in MPL
        for cmapName in cmapNames:
            if not hasattr(matplotlib.cm, cmapName):
                print('[MplCmapImageView] Unknown cmap name: \'{}\'. Your Matplotlib installation might be outdated.'.format(cmapName))
            else:
                # create a Dictionary just as the one at the top of GradientEditorItem.py
                cmap = getattr(matplotlib.cm, cmapName)
                self.mplCmaps[cmapName] = {'ticks': cmapToColormap(cmap), 'mode': 'rgb'}

                # Create the menu entries
                # The following code is copied from pyqtgraph.ImageView.__init__() ...
                px = QtGui.QPixmap(100, 15)
                p = QtGui.QPainter(px)
                self.gradientEditorItem.restoreState(self.mplCmaps[cmapName])
                grad = self.gradientEditorItem.getGradient()
                brush = QtGui.QBrush(grad)
                p.fillRect(QtCore.QRect(0, 0, 100, 15), brush)
                p.end()
                label = QtGui.QLabel()
                label.setPixmap(px)
                label.setContentsMargins(1, 1, 1, 1)
                act = QtGui.QWidgetAction(self.gradientEditorItem)
                act.setDefaultWidget(label)
                act.triggered.connect(self.cmapClicked)
                act.name = cmapName
                self.gradientEditorItem.menu.addAction(act)
        self.gradientEditorItem.length = savedLength
        


    def cmapClicked(self, b=None):
        """onclick handler for our custom entries in the GradientEditorItem's context menu"""
        act = self.sender()
        self.gradientEditorItem.restoreState(self.mplCmaps[act.name])
        self.activeCm = act.name

    def setColorMap(self, colormap):
        """Set the color map.

        ============= =========================================================
        **Arguments**
        colormap      (A ColorMap() instance) The ColorMap to use for coloring
                      images.
        ============= =========================================================
        """
        self.ui.histogram.gradient.setColorMap(colormap)

    def getProcessedImage(self):
        """Returns the image data after it has been processed by any normalization options in use.
        """
        if self.imageDisp is None:
            image = self.normalize(self.image)
            self.imageDisp = image
            self._imageLevels = self.quickMinMax(self.imageDisp)
            self.levelMin = 0
            self.levelMax = 2
            
        return self.imageDisp


    @addGradientListToDocstring()

    def setPredefinedGradient(self, name):
        """Set one of the gradients defined in :class:`GradientEditorItem <pyqtgraph.graphicsItems.GradientEditorItem>`.
        Currently available gradients are:
        """
        self.ui.histogram.gradient.loadPreset(name)

    def trendToggled(self):
        showRoiPlot = False
        if self.trendAction.isChecked():
            print('showing trendroi')
            showRoiPlot = True
            self.trendroi.show()
            #self.ui.roiPlot.show()
            self.ui.roiPlot.setMouseEnabled(True, True)
            self.ui.splitter.setSizes([self.height()*0.6, self.height()*0.4])
            self.roiCurve.show()
            self.roiChanged()
            self.ui.roiPlot.showAxis('left')
        else:
            self.trendroi.hide()
            self.ui.roiPlot.setMouseEnabled(False, False)
            self.roiCurve.hide()
            self.ui.roiPlot.hideAxis('left')

        if self.hasTimeAxis():
            showRoiPlot = True
            mn = self.tVals.min()
            mx = self.tVals.max()
            self.ui.roiPlot.setXRange(mn, mx, padding=0.01)
            self.timeLine.show()
            self.timeLine.setBounds([mn, mx])
            self.ui.roiPlot.show()
            if not self.trendAction.isChecked():
                self.ui.splitter.setSizes([self.height()-35, 35])
        else:
            self.timeLine.hide()
            #self.ui.roiPlot.hide()

        self.ui.roiPlot.setVisible(showRoiPlot)

    # def normalize(self, image):
    #         """
    #         Process *image* using the normalization options configured in the
    #         control panel.

    #         This can be repurposed to process any data through the same filter.
    #         """
    #         if self.ui.normOffRadio.isChecked():
    #             return image

    #         div = self.ui.normDivideRadio.isChecked()
    #         norm = image.view(np.ndarray).copy()
    #         #if div:
    #             #norm = ones(image.shape)
    #         #else:
    #             #norm = zeros(image.shape)
    #         if div:
    #             norm = norm.astype(np.float64)

    #         if self.ui.normTimeRangeCheck.isChecked() and image.ndim == 3:
    #             (sind, start) = self.timeIndex(self.normRgn.lines[0])
    #             (eind, end) = self.timeIndex(self.normRgn.lines[1])
    #             #print start, end, sind, eind
    #             #n = image[sind:eind+1].mean(axis=0)
    #             print('averaging time range...')
    #             if eind<sind: #swap order if it is wrong
    #                 sind,eind=eind,sind
    #             n = np.nanmean(image[sind:eind+1],axis=0)
    #             n.shape = (1,) + n.shape
    #             if div:
    #                 print('performing division...')
    #                 norm /= n
    #             else:
    #                 norm=norm.astype(np.float64)
    #                 norm -= n

    #         if self.ui.normFrameCheck.isChecked() and image.ndim == 3:
    #             n = image.mean(axis=1).mean(axis=1)
    #             n.shape = n.shape + (1, 1)
    #             if div:
    #                 norm /= n
    #             else:
    #                 norm -= n

    #         if self.ui.normROICheck.isChecked() and image.ndim == 3:
    #             n = self.normRoi.getArrayRegion(norm, self.imageItem, (1, 2)).mean(axis=1).mean(axis=1)
    #             n = n[:,np.newaxis,np.newaxis]
    #             #print start, end, sind, eind
    #             if div:
    #                 norm /= n
    #             else:
    #                 norm -= n

    #         return norm

    def quickMinMax(self, data):
        """
        Estimate the min/max values of *data* by subsampling.
        """
        while data.size > 1e6:
            ax = np.argmax(data.shape)
            sl = [slice(None)] * data.ndim
            sl[ax] = slice(None, None, 2)
            data = data[sl]
            if data.dtype=='float64':
                data[~np.isfinite(data)] = np.nan
        return np.nanmin(data), np.nanmax(data)

    # def updateNorm(self):
    #     if self.ui.normTimeRangeCheck.isChecked():
    #         self.normRgn.show()
    #     else:
    #         self.normRgn.hide()

    #     if self.ui.normROICheck.isChecked():
    #         self.normRoi.show()
    #     else:
    #         self.normRoi.hide()

    #     if not self.ui.normOffRadio.isChecked():
    #         self.imageDisp = None
    #         self.updateImage(autoHistogramRange=False)
    #         self.autoLevels()
    #         self.roiChanged()
    #         self.sigProcessingChanged.emit(self)


def mkQApp():
    if QtGui.QApplication.instance() is None:
        global QAPP
        QAPP = QtGui.QApplication([])

class ImageWindow(ImageViewPfaff):
    #sigClosed = QtCore.Signal(object)

    """
    (deprecated; use :class:`~pyqtgraph.ImageView` instead)
    """
    def __init__(self, *args, **kargs):
        mkQApp()
        ImageView.__init__(self)
        if 'title' in kargs:
            self.setWindowTitle(kargs['title'])
            del kargs['title']
        if len(args) > 0 or len(kargs) > 0:
            self.setImage(*args, **kargs)
        self.show()

    def closeEvent(self, event):
        ImageView.closeEvent(self, event)
        self.sigClosed.emit(self)


def cmapToColormap(cmap, nTicks=16):
    """
    Converts a Matplotlib cmap to pyqtgraphs colormaps. No dependency on matplotlib.
    Parameters:
    *cmap*: Cmap object. Imported from matplotlib.cm.*
    *nTicks*: Number of ticks to create when dict of functions is used. Otherwise unused.
    """

    # Case #1: a dictionary with 'red'/'green'/'blue' values as list of ranges (e.g. 'jet')
    # The parameter 'cmap' is a 'matplotlib.colors.LinearSegmentedColormap' instance ...
    if hasattr(cmap, '_segmentdata'):
        colordata = getattr(cmap, '_segmentdata')
        if ('red' in colordata) and isinstance(colordata['red'], collections.Sequence):
            # print("[cmapToColormap] RGB dicts with ranges")

            # collect the color ranges from all channels into one dict to get unique indices
            posDict = {}
            for idx, channel in enumerate(('red', 'green', 'blue')):
                for colorRange in colordata[channel]:
                    posDict.setdefault(colorRange[0], [-1, -1, -1])[idx] = colorRange[2]

            indexList = list(posDict.keys())
            indexList.sort()
            # interpolate missing values (== -1)
            for channel in range(3):  # R,G,B
                startIdx = indexList[0]
                emptyIdx = []
                for curIdx in indexList:
                    if posDict[curIdx][channel] == -1:
                        emptyIdx.append(curIdx)
                    elif curIdx != indexList[0]:
                        for eIdx in emptyIdx:
                            rPos = (eIdx - startIdx) / (curIdx - startIdx)
                            vStart = posDict[startIdx][channel]
                            vRange = (posDict[curIdx][channel] - posDict[startIdx][channel])
                            posDict[eIdx][channel] = rPos * vRange + vStart
                        startIdx = curIdx
                        del emptyIdx[:]
            for channel in range(3):  # R,G,B
                for curIdx in indexList:
                    posDict[curIdx][channel] *= 255

            posList = [[i, posDict[i]] for i in indexList]
            return posList

        # Case #2: a dictionary with 'red'/'green'/'blue' values as functions (e.g. 'gnuplot')
        elif ('red' in colordata) and isinstance(colordata['red'], collections.Callable):
            # print("[cmapToColormap] RGB dict with functions")
            indices = np.linspace(0., 1., nTicks)
            luts = [np.clip(np.array(colordata[rgb](indices), dtype=np.float), 0, 1) * 255 \
                    for rgb in ('red', 'green', 'blue')]
            return list(zip(indices, list(zip(*luts))))

    # If the parameter 'cmap' is a 'matplotlib.colors.ListedColormap' instance, with the attributes 'colors' and 'N'
    elif hasattr(cmap, 'colors') and hasattr(cmap, 'N'):
        colordata = getattr(cmap, 'colors')
        # Case #3: a list with RGB values (e.g. 'seismic')
        if len(colordata[0]) == 3:
            # print("[cmapToColormap] list with RGB values")
            indices = np.linspace(0., 1., len(colordata))
            scaledRgbTuples = [(rgbTuple[0] * 255, rgbTuple[1] * 255, rgbTuple[2] * 255) for rgbTuple in colordata]
            return list(zip(indices, scaledRgbTuples))

        # Case #4: a list of tuples with positions and RGB-values (e.g. 'terrain')
        # -> this section is probably not needed anymore!?
        elif len(colordata[0]) == 2:
            # print("[cmapToColormap] list with positions and RGB-values. Just scale the values.")
            scaledCmap = [(idx, (vals[0] * 255, vals[1] * 255, vals[2] * 255)) for idx, vals in colordata]
            return scaledCmap

    # Case #X: unknown format or datatype was the wrong object type
    else:
        raise ValueError("[cmapToColormap] Unknown cmap format or not a cmap!")

images = []
def image(*args, **kargs):
    """
    Create and return an :class:`ImageWindow <pyqtgraph.ImageWindow>`
    (this is just a window with :class:`ImageView <pyqtgraph.ImageView>` widget inside), show image data inside.
    Will show 2D or 3D image data.
    Accepts a *title* argument to set the title of the window.
    All other arguments are used to show data. (see :func:`ImageView.setImage() <pyqtgraph.ImageView.setImage>`)
    """
    mkQApp()
    w = ImageWindow(*args, **kargs)
    images.append(w)
    w.show()
    return w



import sys
from PyQt5.QtWidgets import QApplication, QWidget, QPushButton, QStyleFactory, \
    QGridLayout, QListWidget, QVBoxLayout, QHBoxLayout, QLineEdit, QListWidgetItem, QStatusBar, QProgressBar
#from PyQt5 import QStyleFactory
from matplotlib.backends.backend_qt5agg \
    import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg \
    import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
import random
import numpy as np
import pyqtgraph as pg
import pyqtgraph.parametertree.parameterTypes as pTypes
from pyqtgraph.parametertree import Parameter, ParameterTree, ParameterItem, registerParameterType
import pyqtgraph.console
import matplotlib.image as mpimg
from os import listdir
from os.path import isfile, join
from camview.readimg import sbf, plifimg, sif
from camview.readlvfile import readlvfile
from camview.ImageViewPfaff import ImageViewPfaff
import pickle

class LVView(pg.GraphicsWindow):

    plots=[] #list of all plots
    sliders=[]
    def __init__(self,path='./testdata/2021-03-17/2021-03-17_real27.txt'):
        super().__init__()
        self.makeLVWindow(path)
        
    def LVtimeLineChanged(self, ev):
        for s in self.sliders:
            if s == ev:
                continue
            else:
                s.setPos(ev.getPos())

    def makeLVWindow(self,lvpath):
        #load LV data
        timeline,unitline,times,temps,currents,pressures,flows,msdata=readlvfile(lvpath)
        
        sepunits=unitline.split()

        #define window stuff
        pens=['r','b','g','y','c']
        self.resize(1000,600)
        self.setWindowTitle(lvpath)

        #first plot
        p1 = self.addPlot(title="Heating")
        p1.addLegend()
        p1.plot(times,temps,pen=(0,255,0),name='Temperature [°C]')
        p1.plot(times,currents*1000,pen=(255,0,0),name='Current [mA]')
        p1.setLabel('bottom',text='time',units='s')
        self.plots.append(p1)

        #second plot
        self.nextRow()
        p2 = self.addPlot(title="Flows")
        for i in range(0,5):
            p2.plot(times,flows[:,i],pen=pens[i],name=sepunits[i+11].split('[')[0])
        p2.setXLink(p1)
        p2.addLegend()
        self.plots.append(p2)

        #third plot
        self.nextRow()
        p3 = self.addPlot(title="MS")
        for i in range(0,5):
            p3.plot(times,np.log10(msdata[:,i]),pen=pens[i])
        p3.setXLink(p1)
        self.plots.append(p3)

        for plot in self.plots:
            timelineSlider=pg.InfiniteLine(0, movable=True)
            plot.addItem(timelineSlider)
            timelineSlider.sigPositionChanged.connect(self.LVtimeLineChanged)
            self.sliders.append(timelineSlider)
        self.trendScroll=self.sliders[0]
    def getWindow(self):
        return self.lvwin


class LVViewApp:
    def __init__(self):
        import os
        os.environ["QT_AUTO_SCREEN_SCALE_FACTOR"] = "1"
        app = QApplication(sys.argv)
        if len(sys.argv) > 1:
            win = LVView(sys.argv[1])
        else:
            win = LVView()
        app.setApplicationName('LU PLIF Viewer')
        #app.setWindowIcon(QIcon('PLIF-icon-256.png'))
        # sys.exit(app.exec_())
        app.exec_()

if __name__ == '__main__':
    app=LVViewApp()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jun 22 16:11:07 2018

@author: sebastian
"""
import numpy as np

def readlvfile(filename):
    lvfile = open(filename)
    lvlines=lvfile.readlines()
    timeline=lvlines[0]
    unitline=lvlines[1]
    
    #remove top two lines
    lvlines.pop(0)
    lvlines.pop(0)
    
    filelen=len(lvlines)
    
    columns=[]
    
    for line in lvlines:
        line=line.strip()
        columns.append(line.split())
    
    #define rows
    
    times=np.empty(filelen)
    temps=np.empty(filelen)
    currents=np.empty(filelen)
    
    pressures=np.empty((filelen,4))
    flows=np.empty((filelen,5))
    msdata=np.empty((filelen,5))
    
    #time,temp,current
    for i in range(filelen):
        times[i]=columns[i][0]
        temps[i]=columns[i][1]
        currents[i]=columns[i][2]
        for j in range(3,7):
            pressures[i,j-3]=float(columns[i][j])
        for j in range(7,12):
            flows[i,j-7]=columns[i][j]
        if len(columns[i]) > 12:
            for j in range(12,17):
                msdata[i,j-12]=float(columns[i][j])
        
    return (timeline,unitline,times,temps,currents,pressures,flows,msdata)


def readlvfile2(filename):
    lvfile = open(filename)
    lvlines=lvfile.readlines()
    timeline=lvlines[0]
    unitline=lvlines[1]
    
    #remove top two lines
    lvlines.pop(0)
    lvlines.pop(0)
    
    filelen=len(lvlines)
    
    columns=[]
    
    for line in lvlines:
        line=line.strip()
        columns.append(line.split())
    
    #define rows
    
    times=np.empty(filelen)
    temps=np.empty(filelen)
    currents=np.empty(filelen)
    pressures=np.empty((filelen,4))
    flows=np.empty((filelen,5))
    msdata=np.empty((filelen,6))
    heats=np.empty(filelen)
    steppers=np.empty(filelen)
    sor=np.empty(filelen)
    
    #time,temp,current
    for i in range(filelen):
        times[i]=columns[i][0]
        temps[i]=columns[i][1]
        currents[i]=columns[i][2]
        for j in range(3,7):
            pressures[i,j-3]=float(columns[i][j])
        for j in range(7,14):
            flows[i,j-7]=columns[i][j]
        if len(columns[i]) > 15:
            for j in range(14,20):
                msdata[i,j-12]=float(columns[i][j])
        heats[i]=columns[i][20]
        steppers[i]=columns[i][21]
        sor[i]=columns[i][21]
        
    return (timeline,unitline,times,temps,currents,pressures,flows,msdata,heats,steppers,sor)

def readmaestrofile(filename):
    maestrofile = open(filename)
    maestrolines=maestrofile.readlines()

    
    filelen=len(maestrolines)
    energies=np.empty(filelen)
        
    #for line in maestrolines:
    #    line=line.strip()
        
    for i in range(filelen):
        energies[i]=float(maestrolines[i])

    return (energies)


def readlvfile2dict(filename):
    lvfile = open(filename)
    lvlines=lvfile.readlines()
    lvInfo={}
    lvInfo['timeline']=lvlines[0].rstrip()
    lvInfo['unitline']=lvlines[1]
    
    #remove top two lines
    lvlines.pop(0)
    lvlines.pop(0)
    
    filelen=len(lvlines)
    
    columns=[]
    
    for line in lvlines:
        line=line.strip()
        columns.append(line.split())
    
    #define rows
    lvInfo['times']=np.empty(filelen)
    lvInfo['temps']=np.empty(filelen)
    lvInfo['currents']=np.empty(filelen)
    lvInfo['pressures']=np.empty((filelen,4))
    lvInfo['flows']=np.empty((filelen,5))
    lvInfo['msdata']=np.empty((filelen,6))
    lvInfo['heats']=np.empty(filelen)
    lvInfo['steppers']=np.empty(filelen)
    
    #time,temp,current
    for i in range(filelen):
        lvInfo['times'][i]=columns[i][0]
        lvInfo['temps'][i]=columns[i][1]
        lvInfo['currents'][i]=columns[i][2]
        for j in range(3,7):
            lvInfo['pressures'][i,j-3]=float(columns[i][j])
        for j in range(7,12):
            lvInfo['flows'][i,j-7]=columns[i][j]
        if len(columns[i]) > 12:
            for j in range(12,18):
                lvInfo['msdata'][i,j-12]=float(columns[i][j])
        if len(columns[i]) > 19:
            lvInfo['heats'][i]=columns[i][18]
            lvInfo['steppers'][i]=columns[i][19]
        elif len(columns[i]) > 17:
            lvInfo['heats'][i]=columns[i][17]
            lvInfo['steppers'][i]=columns[i][18]
        
    return (lvInfo)


''' winspec.py - read SPE files created by WinSpec with Princeton Instruments' cameras. '''

import ctypes, os
import struct
import numpy as np
import logging

__all__ = ['SpeFile', 'print_offsets']

__author__ = "Anton Loukianov"
__email__ = "anton.loukianov@gmail.com"
__license__ = "BSD"
__version__ = "0.2.1"

log = logging.getLogger('winspec')

# Definitions of types
spe_byte = ctypes.c_ubyte
spe_word = ctypes.c_ushort
spe_dword = ctypes.c_uint

spe_char = ctypes.c_char # 1 byte
spe_short = ctypes.c_short # 2 bytes

# long is 4 bytes in the manual. It is 8 bytes on my machine
spe_long = ctypes.c_int # 4 bytes

spe_float = ctypes.c_float # 4 bytes
spe_double = ctypes.c_double # 8 bytes

class ROIinfo(ctypes.Structure):
    pass

class AxisCalibration(ctypes.Structure):
    pass

class Header(ctypes.Structure):
    pass

def print_offsets():
    ''' Print the attribute names, sizes and offsets in the C structure
    
    Assuming that the sizes are correct and add up to an offset of 4100 bytes, 
    everything should add up correctly. This information was taken from the 
    WinSpec 2.6 Spectroscopy Software User Manual version 2.6B, page 251.

    If this table doesn't add up, something changed in the definitions of the 
    datatype widths. Fix this in winspec.structs file and let me know!
    '''

    import inspect, re

    A = Header()

    for i in [Header, AxisCalibration, ROIinfo]:
        fields = []

        print('\n{:30s}[{:4s}]\tsize'.format(repr(i), 'offs'))
        
        for name,obj in inspect.getmembers(i):
            if inspect.isdatadescriptor(obj) and not inspect.ismemberdescriptor(obj) \
                and not inspect.isgetsetdescriptor(obj):
                
                fields.append((name, obj))

        fields = sorted(fields, key=lambda x: x[1].offset)

        for name, obj in fields:
            print('{:30s}[{:4d}]\t{:4d}'.format(name, obj.size, obj.offset))


class SpeFile(object):
    ''' A file that represents the SPE file.

    All details written in the file are contained in the `header` structure. Data is 
    accessed by using the `data` property.

    Once the object is created and data accessed, the file is NOT read again. Create
    a new object if you want to reread the file.
    '''

    # Map between header datatype field and numpy datatype 
    _datatype_map = {0 : np.float32, 1 : np.int32, 2 : np.int16, 3 : np.uint16}

    def __init__(self, name):
        ''' Open file `name` to read the header.'''

        with open(name, mode='rb') as f:
            self.header = Header()
            self.path = os.path.realpath(name) 
            self._data = None
            self._xaxis = None
            self._yaxis = None

            # Deprecated method, but FileIO apparently can't be used with numpy
            f.readinto(self.header)

        # set some useful properties
        self.reversed = True if self.header.geometric == 2 else False
        self.gain = self.header.gain

        if self.header.ADCtype == 8:
            self.adc = 'Low Noise'
        elif self.header.ADCtype == 9:
            self.adc = 'High Capacity'
        else:
            self.adc = 'Unknown'

        if self.header.ADCrate == 12:
            self.adc_rate = '2 MHz'
        elif self.header.ADCrate == 6:
            self.adc_rate = '100 KHz'
        else:
            self.adc_rate = 'Unknown'

        self.readout_time = self.header.ReadoutTime

    def _read(self):
        ''' Read the data segment of the file and create an appropriately-shaped numpy array

        Based on the header, the right datatype is selected and returned as a numpy array.  I took 
        the convention that the frame index is the first, followed by the x,y coordinates.
        '''

        if self._data is not None:
            log.debug('using cached data')
            return self._data

        # In python 2.7, apparently file and FileIO cannot be used interchangably
        with open(self.path, mode='rb') as f:
            f.seek(4100) # Skip header (4100 bytes)

            _count = self.header.xdim * self.header.ydim * self.header.NumFrames
            
            self._data = np.fromfile(f, dtype=SpeFile._datatype_map[self.header.datatype], count=_count)

            # Also, apparently the ordering of the data corresponds to how it is stored by the shift register
            # Thus, it appears a little backwards...
            self._data = self._data.reshape((self.header.NumFrames, self.header.ydim, self.header.xdim))

            # Orient the structure so that it is indexed like [NumFrames][x, y]
            self._data = np.rollaxis(self._data, 2, 1)

            # flip data
            if all([self.reversed == True, self.adc == '100 KHz']):
                pass
            elif any([self.reversed == True, self.adc == '100 KHz']):
                self._data = self._data[:, ::-1, :]
                log.debug('flipped data because of nonstandard ADC setting ' +\
                        'or reversed setting')

            return self._data

    @property
    def xaxis(self):
        if self._xaxis is not None:
            log.debug('using cached xaxis')
            return self._xaxis

        px, py = self._make_axes()

        return px

    @property
    def yaxis(self):
        if self._yaxis is not None:
            log.debug('using cached yaxis')
            return self._yaxis

        px, py = self._make_axes()

        return py

    @property
    def xaxis_label(self):
        '''Read the x axis label
        '''
        return self.header.xcalibration.string.decode('ascii')

    @property
    def yaxis_label(self):
        '''Read the y axis label
        '''
        return self.header.ycalibration.string.decode('ascii')


    def _make_axes(self):
        '''Construct axes from calibration fields in header file
        '''
        xcalib = self.header.xcalibration
        ycalib = self.header.ycalibration

        xcalib_valid = struct.unpack('?', xcalib.calib_valid)

        if xcalib_valid:
            xcalib_order, = struct.unpack('>B', xcalib.polynom_order) # polynomial order
            px = xcalib.polynom_coeff[:xcalib_order+1]
            px = np.array(px[::-1]) # reverse coefficients to use numpy polyval
            pixels = np.arange(1, self.header.xdim + 1)
            px = np.polyval(px, pixels)
        else:
            px = np.arange(1, self.header.xdim + 1)

        ycalib_valid = struct.unpack('?', ycalib.calib_valid)

        if ycalib_valid:
            ycalib_order, = struct.unpack('>B', ycalib.polynom_order) # polynomial order
            py = ycalib.polynom_coeff[:ycalib_order+1]
            py = np.array(py[::-1]) # reverse coefficients to use numpy polyval
            pixels = np.arange(1, self.header.ydim + 1)
            py = np.polyval(py, pixels)
        else:
            py = np.arange(1, self.header.ydim + 1)

        self._xaxis = px
        self._yaxis = py

        return px, py


    ''' Data recorded in the file, returned as a numpy array. 
    
    The convention for indexes is that the first index is the frame index, followed by x,y region of 
    interest.
    '''
    data = property(fget=_read)

    def __str__(self):
        return 'SPE File \n\t{:d}x{:d} area, {:d} frames\n\tTaken on {:s}' \
                .format(self.header.xdim, self.header.ydim, 
                        self.header.NumFrames, self.header.date.decode())

    def __repr__(self):
        return str(self)


# Lengths of arrays used in header
HDRNAMEMAX = 120
USERINFOMAX = 1000
COMMENTMAX = 80
LABELMAX = 16
FILEVERMAX = 16
DATEMAX = 10
ROIMAX = 10
TIMEMAX = 7

# Definitions of WinSpec structures

# Region of interest defs
ROIinfo._pack_ = 1
ROIinfo._fields_ = [
    ('startx', spe_word), 
    ('endx', spe_word),
    ('groupx', spe_word),
    ('starty', spe_word),
    ('endy', spe_word),
    ('groupy', spe_word)]

# Calibration structure for X and Y axes
AxisCalibration._pack_ = 1
AxisCalibration._fields_ = [
    ('offset', spe_double), 
    ('factor', spe_double),
    ('current_unit', spe_char),
    ('reserved1', spe_char),
    ('string', spe_char * 40),
    ('reserved2', spe_char * 40), 
    ('calib_valid', spe_char),
    ('input_unit', spe_char),
    ('polynom_unit', spe_char),
    ('polynom_order', spe_char),
    ('calib_count', spe_char),
    ('pixel_position', spe_double * 10),
    ('calib_value', spe_double * 10), 
    ('polynom_coeff', spe_double * 6),
    ('laser_position', spe_double),
    ('reserved3', spe_char),
    ('new_calib_flag', spe_byte),
    ('calib_label', spe_char * 81),
    ('expansion', spe_char * 87)]

# Full header definition
Header._pack_ = 1
Header._fields_ = [
    ('ControllerVersion', spe_short),
    ('LogicOutput', spe_short),
    ('AmpHiCapLowNoise', spe_word),
    ('xDimDet', spe_word),
    ('mode', spe_short),
    ('exp_sec', spe_float),
    ('VChipXdim', spe_short),
    ('VChipYdim', spe_short),
    ('yDimDet', spe_word),
    ('date', spe_char * DATEMAX),
    ('VirtualChipFlag', spe_short),
    ('Spare_1', spe_char * 2), # Unused data
    ('noscan', spe_short),
    ('DetTemperature', spe_float),
    ('DetType', spe_short),
    ('xdim', spe_word),
    ('stdiode', spe_short),
    ('DelayTime', spe_float),
    ('ShutterControl', spe_word),
    ('AbsorbLive', spe_short),
    ('AbsorbMode', spe_word),
    ('CanDoVirtualChipFlag', spe_short),
    ('ThresholdMinLive', spe_short),
    ('ThresholdMinVal', spe_float), 
    ('ThresholdMaxLive', spe_short), 
    ('ThresholdMaxVal', spe_float),
    ('SpecAutoSpectroMode', spe_short),
    ('SpecCenterWlNm', spe_float),
    ('SpecGlueFlag', spe_short),
    ('SpecGlueStartWlNm', spe_float),
    ('SpecGlueEndWlNm', spe_float),
    ('SpecGlueMinOvrlpNm', spe_float),
    ('SpecGlueFinalResNm', spe_float),
    ('PulserType', spe_short),
    ('CustomChipFlag', spe_short),
    ('XPrePixels', spe_short),
    ('XPostPixels', spe_short),
    ('YPrePixels', spe_short),
    ('YPostPixels', spe_short),
    ('asynen', spe_short),
    ('datatype', spe_short), # 0 - float, 1 - long, 2 - short, 3 - ushort
    ('PulserMode', spe_short),
    ('PulserOnChipAccums', spe_word),
    ('PulserRepeatExp', spe_dword),
    ('PulseRepWidth', spe_float),
    ('PulseRepDelay', spe_float),
    ('PulseSeqStartWidth', spe_float),
    ('PulseSeqEndWidth', spe_float),
    ('PulseSeqStartDelay', spe_float),
    ('PulseSeqEndDelay', spe_float),
    ('PulseSeqIncMode', spe_short),
    ('PImaxUsed', spe_short),
    ('PImaxMode', spe_short),
    ('PImaxGain', spe_short),
    ('BackGrndApplied', spe_short),
    ('PImax2nsBrdUsed', spe_short),
    ('minblk', spe_word),
    ('numminblk', spe_word),
    ('SpecMirrorLocation', spe_short * 2),
    ('SpecSlitLocation', spe_short * 4),
    ('CustomTimingFlag', spe_short),
    ('ExperimentTimeLocal', spe_char * TIMEMAX),
    ('ExperimentTimeUTC', spe_char * TIMEMAX),
    ('ExposUnits', spe_short),
    ('ADCoffset', spe_word),
    ('ADCrate', spe_word),
    ('ADCtype', spe_word),
    ('ADCresolution', spe_word),
    ('ADCbitAdjust', spe_word),
    ('gain', spe_word),
    ('Comments', spe_char * 5 * COMMENTMAX),
    ('geometric', spe_word), # x01 - rotate, x02 - reverse, x04 flip
    ('xlabel', spe_char * LABELMAX),
    ('cleans', spe_word),
    ('NumSkpPerCln', spe_word),
    ('SpecMirrorPos', spe_short * 2),
    ('SpecSlitPos', spe_float * 4), 
    ('AutoCleansActive', spe_short),
    ('UseContCleansInst', spe_short),
    ('AbsorbStripNum', spe_short), 
    ('SpecSlipPosUnits', spe_short),
    ('SpecGrooves', spe_float),
    ('srccmp', spe_short),
    ('ydim', spe_word), 
    ('scramble', spe_short),
    ('ContinuousCleansFlag', spe_short), 
    ('ExternalTriggerFlag', spe_short), 
    ('lnoscan', spe_long), # Longs are 4 bytes  
    ('lavgexp', spe_long), # 4 bytes
    ('ReadoutTime', spe_float), 
    ('TriggeredModeFlag', spe_short), 
    ('Spare_2', spe_char * 10), 
    ('sw_version', spe_char * FILEVERMAX), 
    ('type', spe_short),
    ('flatFieldApplied', spe_short), 
    ('Spare_3', spe_char * 16), 
    ('kin_trig_mode', spe_short), 
    ('dlabel', spe_char * LABELMAX), 
    ('Spare_4', spe_char * 436), 
    ('PulseFileName', spe_char * HDRNAMEMAX), 
    ('AbsorbFileName', spe_char * HDRNAMEMAX),
    ('NumExpRepeats', spe_dword),
    ('NumExpAccums', spe_dword),
    ('YT_Flag', spe_short), 
    ('clkspd_us', spe_float),
    ('HWaccumFlag', spe_short),
    ('StoreSync', spe_short),
    ('BlemishApplied', spe_short),
    ('CosmicApplied', spe_short), 
    ('CosmicType', spe_short),
    ('CosmicThreshold', spe_float), 
    ('NumFrames', spe_long),
    ('MaxIntensity', spe_float),
    ('MinIntensity', spe_float),
    ('ylabel', spe_char * LABELMAX),
    ('ShutterType', spe_word),
    ('shutterComp', spe_float),
    ('readoutMode', spe_word),
    ('WindowSize', spe_word),
    ('clkspd', spe_word),
    ('interface_type', spe_word),
    ('NumROIsInExperiment', spe_short),
    ('Spare_5', spe_char * 16),
    ('controllerNum', spe_word),
    ('SWmade', spe_word),
    ('NumROI', spe_short),
    ('ROIinfblk', ROIinfo * ROIMAX),
    ('FlatField', spe_char * HDRNAMEMAX),
    ('background', spe_char * HDRNAMEMAX),
    ('blemish', spe_char * HDRNAMEMAX),
    ('file_header_ver', spe_float),
    ('YT_Info', spe_char * 1000),
    ('WinView_id', spe_long),
    ('xcalibration', AxisCalibration),
    ('ycalibration', AxisCalibration),
    ('Istring', spe_char * 40),
    ('Spare_6', spe_char * 25),
    ('SpecType', spe_byte),
    ('SpecModel', spe_byte),
    ('PulseBurstUsed', spe_byte),
    ('PulseBurstCount', spe_dword),
    ('PulseBurstPeriod', spe_double),
    ('PulseBracketUsed', spe_byte),
    ('PulseBracketType', spe_byte),
    ('PulseTimeConstFast', spe_double),
    ('PulseAmplitudeFast', spe_double),
    ('PulseTimeConstSlow', spe_double),
    ('PulseAmplitudeSlow', spe_double),
    ('AnalogGain', spe_short),
    ('AvGainUsed', spe_short),
    ('AvGain', spe_short),
    ('lastvalue', spe_short)]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Dec 19 13:41:45 2017

@author: sebastian
"""
import sys
from PyQt5.QtWidgets import QApplication, QWidget, QPushButton, QStyleFactory, \
    QGridLayout, QListWidget, QVBoxLayout, QHBoxLayout, QLineEdit, QListWidgetItem, \
        QStatusBar, QProgressBar, QSplitter, QGroupBox, QCheckBox, QRadioButton, QComboBox, QLineEdit,\
            QLabel, QFormLayout, QSpinBox, QFileDialog
#from PyQt5 import QStyleFactory
from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtGui import QIcon


import numpy as np
import pyqtgraph as pg
import matplotlib.image as mpimg
from os import listdir
import os
from os.path import isfile, join
from camview.readimg import sbf, plifimg, sif, lvsor, mptif16
from camview.readlvfile import readlvfile
from camview.ImageViewPfaff import ImageViewPfaff
import pickle
import pyqtgraph.ptime as ptime

class CCCView(QWidget):
    def __init__(self,LoadFile=None):
        super().__init__()
        self.LoadFileName=LoadFile
        self.noRepeatKeys = [QtCore.Qt.Key_Right, QtCore.Qt.Key_Left, QtCore.Qt.Key_Up, QtCore.Qt.Key_Down, QtCore.Qt.Key_PageUp, QtCore.Qt.Key_PageDown]
        self.keysPressed = {}
        self.playTimer = QtCore.QTimer()
        self.playRate = 0
        self.fps = 1 # 1 Hz by default
        self.lastPlayTime = 0
        self.playTimer.timeout.connect(self.timeout)

        self.PLIFFileName=''
        self.readRaw=False
        self.fromFrame=0
        self.toFrame=-1
        self.profileMode=0
        self.profileData=None

        self.dispInvert=False
        self.dispDiff=False
        self.divProfile=True
        self.showData=np.random.rand(256,256,256)
        self.averagedProfiles={}
        self.childViews=[]
        self.path = '.'
        #self.initUI()

        if len(sys.argv) > 1:
            self.path = sys.argv[1]
        if self.LoadFileName is not None:
            print(self.LoadFileName)
            self.path=os.path.dirname(self.LoadFileName)

    def cmapClicked(self,LoadFile=None):
        """onclick handler for our custom entries in the GradientEditorItem's context menu"""
        act = self.sender()
        self.hist.gradient.restoreState(self.mplCmaps[act.name])
        if act.name == 'turbo':
            self.hist.gradient.showTicks(show=False)

    def initUI(self):
        self.ROIList=[]
        self.timeAxis=None #will contain the time for each frame

        self.resize(1300, 900)
        self.move(500, 300)

        MainGrid=QGridLayout(self)

        win = pg.GraphicsLayoutWidget()

        # self.status = QStatusBar(self)
        # self.statusProgress = QProgressBar()
        # self.status.addWidget(self.statusProgress)
        # MainGrid.addWidget(self.status,20,0,1,2)
        # self.status.showMessage('TestStatus')


        self.imgPlot=win.addPlot(row=0,col=0)
        self.img=pg.ImageItem()
        self.imgPlot.addItem(self.img)
        self.imgPlot.setAspectLocked()

        self.hist = pg.HistogramLUTItem()
        self.mplCmaps = {}

        def registerCmap(gradientEditor, cmapNames):
            import matplotlib
            """ Add matplotlib cmaps to the GradientEditors context menu"""
            #gradientEditor.menu.addSeparator()
            savedLength = gradientEditor.length
            gradientEditor.length = 100

            
            def cmapToColormap(cmap, nTicks=16):
                """
                Converts a Matplotlib cmap to pyqtgraphs colormaps. No dependency on matplotlib.
                Parameters:
                *cmap*: Cmap object. Imported from matplotlib.cm.*
                *nTicks*: Number of ticks to create when dict of functions is used. Otherwise unused.
                """
                import collections
                # Case #1: a dictionary with 'red'/'green'/'blue' values as list of ranges (e.g. 'jet')
                # The parameter 'cmap' is a 'matplotlib.colors.LinearSegmentedColormap' instance ...
                if hasattr(cmap, '_segmentdata'):
                    colordata = getattr(cmap, '_segmentdata')
                    if ('red' in colordata) and isinstance(colordata['red'], collections.Sequence):
                        # print("[cmapToColormap] RGB dicts with ranges")

                        # collect the color ranges from all channels into one dict to get unique indices
                        posDict = {}
                        for idx, channel in enumerate(('red', 'green', 'blue')):
                            for colorRange in colordata[channel]:
                                posDict.setdefault(colorRange[0], [-1, -1, -1])[idx] = colorRange[2]

                        indexList = list(posDict.keys())
                        indexList.sort()
                        # interpolate missing values (== -1)
                        for channel in range(3):  # R,G,B
                            startIdx = indexList[0]
                            emptyIdx = []
                            for curIdx in indexList:
                                if posDict[curIdx][channel] == -1:
                                    emptyIdx.append(curIdx)
                                elif curIdx != indexList[0]:
                                    for eIdx in emptyIdx:
                                        rPos = (eIdx - startIdx) / (curIdx - startIdx)
                                        vStart = posDict[startIdx][channel]
                                        vRange = (posDict[curIdx][channel] - posDict[startIdx][channel])
                                        posDict[eIdx][channel] = rPos * vRange + vStart
                                    startIdx = curIdx
                                    del emptyIdx[:]
                        for channel in range(3):  # R,G,B
                            for curIdx in indexList:
                                posDict[curIdx][channel] *= 255

                        posList = [[i, posDict[i]] for i in indexList]
                        return posList

                    # Case #2: a dictionary with 'red'/'green'/'blue' values as functions (e.g. 'gnuplot')
                    elif ('red' in colordata) and isinstance(colordata['red'], collections.Callable):
                        # print("[cmapToColormap] RGB dict with functions")
                        indices = np.linspace(0., 1., nTicks)
                        luts = [np.clip(np.array(colordata[rgb](indices), dtype=np.float), 0, 1) * 255 \
                                for rgb in ('red', 'green', 'blue')]
                        return list(zip(indices, list(zip(*luts))))

                # If the parameter 'cmap' is a 'matplotlib.colors.ListedColormap' instance, with the attributes 'colors' and 'N'
                elif hasattr(cmap, 'colors') and hasattr(cmap, 'N'):
                    colordata = getattr(cmap, 'colors')
                    # Case #3: a list with RGB values (e.g. 'seismic')
                    if len(colordata[0]) == 3:
                        # print("[cmapToColormap] list with RGB values")
                        indices = np.linspace(0., 1., len(colordata))
                        scaledRgbTuples = [(rgbTuple[0] * 255, rgbTuple[1] * 255, rgbTuple[2] * 255) for rgbTuple in colordata]
                        return list(zip(indices, scaledRgbTuples))

                    # Case #4: a list of tuples with positions and RGB-values (e.g. 'terrain')
                    # -> this section is probably not needed anymore!?
                    elif len(colordata[0]) == 2:
                        # print("[cmapToColormap] list with positions and RGB-values. Just scale the values.")
                        scaledCmap = [(idx, (vals[0] * 255, vals[1] * 255, vals[2] * 255)) for idx, vals in colordata]
                        return scaledCmap

                # Case #X: unknown format or datatype was the wrong object type
                else:
                    raise ValueError("[cmapToColormap] Unknown cmap format or not a cmap!")

            # iterate over the list of cmap names and check if they're avaible in MPL
            for cmapName in cmapNames:
                if not hasattr(matplotlib.cm, cmapName):
                    print('[MplCmapImageView] Unknown cmap name: \'{}\'. Your Matplotlib installation might be outdated.'.format(cmapName))
                else:
                    # create a Dictionary just as the one at the top of GradientEditorItem.py
                    cmap = getattr(matplotlib.cm, cmapName)
                    self.mplCmaps[cmapName] = {'ticks': cmapToColormap(cmap), 'mode': 'rgb'}

                    # Create the menu entries
                    # The following code is copied from pyqtgraph.ImageView.__init__() ...
                    px = QtGui.QPixmap(100, 15)
                    p = QtGui.QPainter(px)
                    gradientEditor.restoreState(self.mplCmaps[cmapName])
                    grad = gradientEditor.getGradient()
                    brush = QtGui.QBrush(grad)
                    p.fillRect(QtCore.QRect(0, 0, 100, 15), brush)
                    p.end()
                    label = QtGui.QLabel()
                    label.setPixmap(px)
                    label.setContentsMargins(1, 1, 1, 1)
                    labelName = QtGui.QLabel(cmapName)
                    hbox = QtGui.QHBoxLayout()
                    hbox.addWidget(labelName)
                    hbox.addWidget(label)
                    widget = QtGui.QWidget()
                    widget.setLayout(hbox)
                    act = QtGui.QWidgetAction(gradientEditor)
                    act.setDefaultWidget(widget)
                    act.triggered.connect(self.cmapClicked)
                    act.name = cmapName
                    gradientEditor.menu.addAction(act)
            gradientEditor.length = savedLength
            gradientEditor.menu.removeAction(gradientEditor.rgbAction)
            gradientEditor.menu.removeAction(gradientEditor.hsvAction)

        registerCmap(self.hist.gradient,['jet','turbo'])

        self.hist.setImageItem(self.img)
        self.hist.setMaximumWidth(200)
        self.hist.gradient.showTicks(show=False)
        self.imgPlot=win.addItem(self.hist,row=0,col=1)


        self.trendPlot=win.addPlot(row=1,col=0)
        self.trendScroll=pg.InfiniteLine(movable=True)
        self.trendPlot.addItem(self.trendScroll)
        self.trendPlot.setMaximumHeight(300)

        MainGrid.addWidget(win,0,0,8,1)

        self.trendScroll.sigPositionChanged.connect(self.sliderDragged)

        if len(sys.argv) > 1:
            self.path = sys.argv[1]

        def pathChanged(text):
            if text[-1]=='/':
                self.path=text
            else:
                self.path=text+'/'
            self.fillPathBoxes(text)


        GlobalGroup=QGroupBox('Global')
        GlobalGroup.setMaximumWidth(400)
        GlobalLayout=QFormLayout(GlobalGroup)
        PathBox=QLineEdit(self.path)
        PathBox.textChanged.connect(pathChanged)
        GlobalLayout.addRow('Path',PathBox)
        MainGrid.addWidget(GlobalGroup,0,1)

        ################ ROI ########################

        ROIGroup = QGroupBox('Region of Interest')
        ROIGrid=QGridLayout(ROIGroup)
        MainGrid.addWidget(ROIGroup,5,1)

        addROIbtn=QPushButton('Add ROI')
        delROIbtn=QPushButton('Delete ROI')
        self.ROIListW=QListWidget()
        #self.ROIListW.setMaximumHeight(100)
        self.ROIListW.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        ROIGrid.addWidget(self.ROIListW,1,0,1,2)
        ROIGrid.addWidget(addROIbtn,0,0)
        ROIGrid.addWidget(delROIbtn,0,1)

        addROIbtn.clicked.connect(self.addROI)
        delROIbtn.clicked.connect(self.delROI)

        OtherData = QGroupBox('Additional Data')
        OtherLayout=QHBoxLayout(OtherData)
        LoadOtherButton=QPushButton('Load LV/SOR/PMIRRAS')
        OtherLayout.addWidget(LoadOtherButton)
        LoadOtherButton.pressed.connect(self.LoadOther)
        MainGrid.addWidget(OtherData,7,1)


        # restore()
        self.show()
        return MainGrid
    def sliderDragged(self):
        """Handles what happens when the time slider is dragged. Synchs will all children."""
        #self.trendScroll.sigPositionChanged.disconnect()
        for child in self.childViews:
            child.trendScroll.setPos(self.trendScroll.getPos()[0])
        #self.trendScroll.sigPositionChanged.connect(self.sliderDragged)
        self.redrawSheet()

    def addROI(self):
        """Handles adding a ROI and creates entry in a tuple containing 
        (The MenuItem in the ROI menu, the ROI element in the plot, the plot showing the trend, and the pen used to draw the ROI and the plot)"""
        numROIs=len(self.ROIList)
        ROIMenuItem=QListWidgetItem('ROI '+str(numROIs))
        ROIMenuItem.setFlags(ROIMenuItem.flags() | QtCore.Qt.ItemIsEditable)
        #ROIMenuItem.setCheckState(True)
        self.ROIListW.addItem(ROIMenuItem)
        actualROI=pg.RectROI((0,0),(20,20))
        ROIPlot=pg.PlotDataItem()
        ROIpen=pg.mkPen(pg.intColor(numROIs))
        ROIPlot.setPen(ROIpen)
        actualROI.setPen(ROIpen)
        appendTuple=(ROIMenuItem,actualROI,ROIPlot,ROIpen)
        self.ROIList.append(appendTuple)
        self.img.getViewBox().addItem(actualROI)
        actualROI.sigRegionChanged.connect(self.ROIDragged)
        self.trendPlot.addItem(ROIPlot)

    def delROI(self):
        for ROIEntry in self.ROIListW.selectedItems():
            for jdx,ROI in enumerate(self.ROIList):
                if ROI[0]==ROIEntry:
                    self.img.getViewBox().removeItem(ROI[1])
                    self.ROIListW.takeItem(self.ROIListW.row(ROIEntry))
                    self.ROIList.pop(jdx)
                    self.trendPlot.removeItem(ROI[2])

    def ROIDragged(self,draggedROI):
        """Handles dragging the ROI. Overload this in the specific class. Otherwise it will just display a trend of showData"""
        dataItem=None
        listItem=None
        for ROI in self.ROIList:
            if ROI[1]==draggedROI:
                dataItem=ROI[2]
                listItem=ROI[0]
        data, coords = draggedROI.getArrayRegion(data=self.showData, img=self.img, axes=(1,2),returnMappedCoords=True) #slice the image
        dataItem.setData(self.timeAxis,np.mean(data,axis=(1,2)))  #plot the ROI trend
        xROILocString='X: {:.0f}-{:.0f}'.format(np.min(coords[0]),np.max(coords[0]))
        yROILocString='Y: {:.0f}-{:.0f}'.format(np.min(coords[1]),np.max(coords[1]))
        listItem.setText(xROILocString+', '+yROILocString)

    def LVtimeLineChanged(self,sender):
        pos=sender.getPos()[0]
        self.trendScroll.setPos(pos)
        
    def LoadPLIF(self):
        """Loads the data and return it in format (time, x, y). Overload it."""
        pass

    def LoadOther(self):
        """Add another file, LV, PLIF, SOR, and in the future PMIRRAS"""
        file=(QFileDialog.getOpenFileName(None,"Select a file...",self.path, filter="SOR Files (*.sor *.tif *.sif);;PLIF Files (*.img *.sif);;LabView Logs (*.txt)"))
        childView=None
        if 'LabView Logs (*.txt)' in file[1]:
            from lvtxtview import LVView
            childView=LVView(file[0])
            for slider in childView.sliders:
                slider.sigPositionChanged.connect(self.LVtimeLineChanged)
            childView.show()
            self.childViews.append(childView)
            return
        if 'SOR Files' in file[1]:
            from pyqtgraphSOR import SORView
            childView=SORView(os.path.dirname(file[0])+'/')
        if 'PLIF Files (*.img *.sif)' in file[1]:
            from PLIFViewer import PLIFView
            childView=PLIFView(os.path.dirname(file[0])+'/')
        childView.trendScroll.sigPositionChanged.connect(self.LVtimeLineChanged)
        childView.show()
        self.childViews.append(childView)


    def setCurrentIndex(self,index):
        """"Move in time. Overload if necessary."""
        self.trendScroll.setPos(index)

    def keyPressEvent(self, ev):
        #print ev.key()
        if ev.key() == QtCore.Qt.Key_Space:
            if self.playRate == 0:
                self.play()
            else:
                self.play(0)
            ev.accept()
        elif ev.key() == QtCore.Qt.Key_Home:
            self.setCurrentIndex(0)
            self.play(0)
            ev.accept()
        elif ev.key() == QtCore.Qt.Key_End:
            self.setCurrentIndex(self.getProcessedImage().shape[0]-1)
            self.play(0)
            ev.accept()
        elif ev.key() in self.noRepeatKeys:
            ev.accept()
            if ev.isAutoRepeat():
                return
            self.keysPressed[ev.key()] = 1
            self.evalKeyState()
        else:
            QtGui.QWidget.keyPressEvent(self, ev)

    def keyReleaseEvent(self, ev):
        if ev.key() in [QtCore.Qt.Key_Space, QtCore.Qt.Key_Home, QtCore.Qt.Key_End]:
            ev.accept()
        elif ev.key() in self.noRepeatKeys:
            ev.accept()
            if ev.isAutoRepeat():
                return
            try:
                del self.keysPressed[ev.key()]
            except:
                self.keysPressed = {}
            self.evalKeyState()
        else:
            QtGui.QWidget.keyReleaseEvent(self, ev)
    
    def jumpFrames(self,n):
        self.trendScroll.setPos(self.trendScroll.getPos()[0]+n)

    def evalKeyState(self):
        if len(self.keysPressed) == 1:
            key = list(self.keysPressed.keys())[0]
            if key == QtCore.Qt.Key_Right:
                self.play(20)
                self.jumpFrames(1)
                self.lastPlayTime = ptime.time() + 0.2  ## 2ms wait before start
                                                        ## This happens *after* jumpFrames, since it might take longer than 2ms
            elif key == QtCore.Qt.Key_Left:
                self.play(-20)
                self.jumpFrames(-1)
                self.lastPlayTime = ptime.time() + 0.2
            elif key == QtCore.Qt.Key_Up:
                self.play(-100)
            elif key == QtCore.Qt.Key_Down:
                self.play(100)
            elif key == QtCore.Qt.Key_PageUp:
                self.play(-1000)
            elif key == QtCore.Qt.Key_PageDown:
                self.play(1000)
        else:
            self.play(0)

    def play(self, rate=None):
        """Begin automatically stepping frames forward at the given rate (in fps).
        This can also be accessed by pressing the spacebar."""
        #print "play:", rate
        if rate is None: 
            rate = self.fps
        self.playRate = rate

        if rate == 0:
            self.playTimer.stop()
            return
            
        self.lastPlayTime = ptime.time()
        if not self.playTimer.isActive():
            self.playTimer.start(16)

    def timeout(self):
        now = ptime.time()
        dt = now - self.lastPlayTime
        if dt < 0:
            return
        n = int(self.playRate * dt)
        if n != 0:
            self.lastPlayTime += (float(n)/self.playRate)
            if int(self.trendScroll.getPos()[0])+n > self.showData.shape[2]:
                self.play(0)
            self.jumpFrames(n)

#%%
#code here is just for testing
if __name__ == '__main__':
    app = QApplication(sys.argv)
    app.setApplicationName('LU PLIF Viewer')
    app.setWindowIcon(QIcon('PLIF-icon-256.png'))
    plifv = CCCView()
    plifv.initUI()
    # sys.exit(app.exec_())
    app.exec_()

# https://medium.com/@joel.barmettler/how-to-upload-your-python-package-to-pypi-65edc5fe9c56
# https://stackoverflow.com/questions/12459811/how-to-embed-matplotlib-in-pyqt-for-dummies
